import re
from collections import defaultdict
from copy import copy
"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""

from faraday_plugins.plugins.plugin import PluginXMLFormat
from faraday_plugins.plugins.plugins_utils import filter_services

import xml.etree.ElementTree as ET

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class OpenvasXmlParser:
    """
    The objective of this class is to parse an xml file generated by the openvas tool.

    TODO: Handle errors.
    TODO: Test openvas output version. Handle what happens if the parser doesn't support it.
    TODO: Test cases.

    @param openvas_xml_filepath A proper xml generated by openvas
    """

    def __init__(self, xml_output, logger):
        self.target = None
        self.port = "80"
        self.host = None
        self.logger = logger
        tree = self.parse_xml(xml_output)
        if tree:
            self.hosts = self.get_hosts(tree)
            self.items = list(self.get_items(tree, self.hosts))
        else:
            self.items = []

    def parse_xml(self, xml_output):
        """
        Open and parse an xml file.

        TODO: Write custom parser to just read the nodes that we need instead of
        reading the whole file.

        @return xml_tree An xml tree instance. None if error.
        """
        try:
            tree = ET.fromstring(xml_output)
        except SyntaxError as err:
            self.logger.error("SyntaxError: %s. %s", err, xml_output)
            return None
        return tree

    def get_items(self, tree, hosts):
        """
        @return items A list of Host instances
        """
        try:
            report = tree.find('report')
            if report:
                results = report.findall('results')
                if results:
                    nodes = report.findall('results')[0]
                else:
                    nodes = tree.findall('result')
            else:
                nodes = tree.findall('result')

            for node in nodes:
                try:
                    yield Item(node, hosts)
                except Exception as e:
                    self.logger.error(f"Error generating Iteem from {node.attrib} [{e}]")

        except Exception as e:
            self.logger.error(f"Tag not found: {e}")

    def get_hosts(self, tree):
        # Hosts are located in: /report/report/host
        # hosts_dict will contain has keys its details and its hostnames
        hosts = tree.findall('report/host')
        hosts_dict = {}
        for host in hosts:
            ip = self.do_clean(host.find('ip').text)
            details = self.get_data_from_detail(host.findall('detail'))
            hosts_dict[ip] = details
        return hosts_dict

    def get_data_from_detail(self, details):
        data = {}
        details_data = defaultdict(list)
        hostnames = []
        for item in details:
            name = self.do_clean(item.find('name').text)
            value = self.do_clean(item.find('value').text)
            if "EXIT" in name:
                continue
            if name == 'hostname':
                hostnames.append(value)
            else:
                details_data[name].append(value)
        data['details'] = details_data
        data['hostnames'] = hostnames
        return data

    def do_clean(self, value):
        myreturn = ""
        if value is not None:
            myreturn = re.sub(r"\s+", " ", value)
        return myreturn.strip()


def get_attrib_from_subnode(xml_node, subnode_xpath_expr, attrib_name):
    """
    Finds a subnode in the item node and the retrieves a value from it

    @return An attribute value
    """

    node = xml_node.find(subnode_xpath_expr)

    if node is not None:
        return node.get(attrib_name)

    return None


class Item:
    """
    An abstract representation of a Item
    @param item_node A item_node taken from an openvas xml tree
    """

    def __init__(self, item_node, hosts):
        self.node = item_node
        self.host = self.get_text_from_subnode('host')
        self.threat = self.get_text_from_subnode('threat')
        self.subnet = self.get_text_from_subnode('subnet')
        if self.subnet == '':
            self.subnet = self.host
        self.port = None
        self.severity = self.severity_mapper()
        self.severity_nr = self.get_text_from_subnode("severity")
        self.service = "Unknown"
        self.protocol = ""
        details = self.node.findall("detection/result/details/detail")
        self.cpe = details[0].findtext("value") if details else None
        port_string = self.get_text_from_subnode('port')
        info = port_string.split("/")
        if len(info) < 2:
            info = details[1].findtext("value").split("/") if details else ["", ""]
        self.protocol = "".join(filter(lambda x: x.isalpha() or x in ("-", "_"), info[1]))
        self.port = "".join(filter(lambda x: x.isdigit(), info[0])) or None
        if not self.port:
            self.service = info[0]
        else:
            if hosts:
                host_details = hosts[self.host].get('details')
                self.service = self.get_service(port_string, self.port, host_details)
            else:
                self.service = "Not Service"
        self.nvt = self.node.findall('nvt')[0]
        self.node = self.nvt
        self.id = self.node.get('oid')
        self.cvss_base = self.get_text_from_subnode("cvss_base")
        self.name = self.get_text_from_subnode('name')
        self.cve = self.get_text_from_subnode('cve') if self.get_text_from_subnode('cve') != "NOCVE" else ""
        self.bid = self.get_text_from_subnode('bid') if self.get_text_from_subnode('bid') != "NOBID" else ""
        self.xref = self.get_text_from_subnode('xref') if self.get_text_from_subnode('xref') != "NOXREF" else ""
        self.cwe = []
        if "URL:https://cwe.mitre.org/data/definitions/" in self.xref:
            self.cwe.append("CWE-"+self.xref.split("URL:https://cwe.mitre.org/data/definitions/")[1]
                            .replace("html", ""))
        self.description = ''
        self.resolution = ''
        self.cvss_vector = ''
        self.tags = self.get_text_from_subnode('tags')
        self.data = self.get_text_from_subnode('description')
        self.data += f'\n\nid {item_node.attrib.get("id")}'
        if self.tags:
            tags_data = self.get_data_from_tags(self.tags)
            self.description = tags_data['description']
            self.resolution = tags_data['solution']
            self.cvss_vector = tags_data['cvss_base_vector']
            if tags_data['impact']:
                self.data += f'\n\nImpact: {tags_data["impact"]}'

    def get_text_from_subnode(self, subnode_xpath_expr):
        """
        Finds a subnode in the host node and the retrieves a value from it.

        @return An attribute value
        """
        sub_node = self.node.find(subnode_xpath_expr)
        if sub_node is not None and sub_node.text is not None:
            return sub_node.text.strip()
        return ''

    def severity_mapper(self):
        severity = self.get_text_from_subnode('threat')
        if severity == 'Alarm':
            severity = 'Critical'
        return severity

    def get_service(self, port_string, port, details_from_host):
        # details_from_host:
        # name: name of detail
        # value: list with the values associated with the name
        details_from_host_copy = copy(details_from_host)
        services = details_from_host_copy.pop("Services", None)
        if services:
            service_detail = self.get_service_from_details("Services", services, port)
            if service_detail:
                return service_detail
        for name, value in details_from_host_copy.items():
            service_detail = self.get_service_from_details(name, value, port)
            if service_detail:
                return service_detail
        # if the service is not in details_from_host, we will search it in
        # the file port_mapper.txt
        services_mapper = filter_services()
        for service in services_mapper:
            if service[0] == port_string:
                return service[1]
        return "Unknown"

    def do_clean(self, value):
        myreturn = ""
        if value is not None:
            myreturn = re.sub(r"\s+", " ", value)

        return myreturn.strip()

    def get_service_from_details(self, name, value_list, port):
        # detail:
        # name: name of detail
        # value_list: list with the values associated with the name
        res = None
        priority = 0
        if name == 'Services':
            for value in value_list:
                value_splited = value.split(',')
                if value_splited[0] == port:
                    res = value_splited[2]
                    break
        else:
            for value in value_list:
                if '/' in value:
                    auxiliar_value = value.split('/')[0]
                    if auxiliar_value == port:
                        res = name
                        priority = 2

                elif value.isdigit() and priority == 0:
                    if value == port:
                        res = name
                        priority = 1

                elif '::' in value and priority == 0:
                    aux_value = value.split('::')[0]
                    if aux_value == port:
                        res = name
        return res

    def get_data_from_tags(self, tags_text):
        clean_text = self.do_clean(tags_text)
        tags = clean_text.split('|')
        summary = ''
        insight = ''
        data = {
            'solution': '',
            'cvss_base_vector': '',
            'description': '',
            'impact': ''
        }
        for tag in tags:
            splited_tag = tag.split('=', 1)
            if splited_tag[0] in data.keys():
                data[splited_tag[0]] = splited_tag[1]
            elif splited_tag[0] == 'summary':
                summary = splited_tag[1]
            elif splited_tag[0] == 'insight':
                insight = splited_tag[1]

        data['description'] = ' '.join([summary, insight]).strip()

        return data


class OpenvasPlugin(PluginXMLFormat):
    """
    Example plugin to parse openvas output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = ["report", "get_results_response"]
        self.id = "Openvas"
        self.name = "Openvas XML Output Plugin"
        self.plugin_version = "0.3"
        self.version = "9.0.3"
        self.framework_version = "1.0.0"
        self.options = None

    def report_belongs_to(self, **kwargs):
        if super().report_belongs_to(**kwargs):
            report_path = kwargs.get("report_path", "")
            with open(report_path) as f:
                output = f.read()
            return re.search("OpenVAS", output) is not None \
                   or re.search('<omp>', output) is not None \
                   or re.search('<owner>', output) is not None
        return False

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it
        from the xml where it expects it to be present.
        """
        parser = OpenvasXmlParser(output, self.logger)
        ids = {}
        # The following threats values will not be taken as vulns
        self.ignored_severities = ['Log', 'Debug']
        for ip, values in parser.hosts.items():
            # values contains: ip details and ip hostnames
            os_report = values['details'].get('best_os_txt')
            h_id = self.createAndAddHost(
                ip,
                os_report[0] if os_report else '',
                hostnames=values['hostnames']
            )
            ids[ip] = h_id
        for item in parser.items:

            if item.name is not None:
                ref = []
                cve = []
                cvss2 = {}
                if item.cve:
                    cves = item.cve.split(',')
                    for i in cves:
                        cve.append(i.strip())
                if item.bid:
                    bids = item.bid.split(',')
                    for bid in bids:
                        ref.append(f"BID-{bid.strip()}")
                if item.xref:
                    ref.append(item.xref)
                if item.tags and item.cvss_vector:
                    cvss2["vector_string"] = item.cvss_vector
                if item.cpe:
                    ref.append(f"{item.cpe}")
                if item.severity_nr:
                    if float(item.severity_nr) >= 9.0:
                        item.threat = "Critical"
                        item.severity = "Critical"
                    ref.append(f"SEVERITY NUMBER: {item.severity_nr}")
                if item.threat:
                    ref.append(f"THREAT: {item.threat}")

                if item.subnet in ids:
                    h_id = ids[item.host]
                else:
                    h_id = self.createAndAddHost(
                        item.subnet,
                        hostnames=[item.host])
                    ids[item.subnet] = h_id

                if not item.port:
                    if item.severity not in self.ignored_severities:
                        v_id = self.createAndAddVulnToHost(
                            h_id,
                            item.name,
                            desc=item.description,
                            severity=item.severity,
                            resolution=item.resolution,
                            ref=ref,
                            external_id=f"OPENVAS-{item.id}",
                            data=item.data,
                            cve=cve,
                            cwe=item.cwe,
                            cvss2=cvss2
                        )
                else:
                    if item.service:
                        web = re.search(
                            r'^(www|http)',
                            item.service)
                    else:
                        web = item.port in ('80', '443', '8080')

                    if item.subnet + "_" + item.port in ids:
                        s_id = ids[item.subnet + "_" + item.port]
                    else:
                        s_id = self.createAndAddServiceToHost(
                            h_id,
                            item.service,
                            item.protocol,
                            ports=[str(item.port)]
                        )
                        ids[item.subnet + "_" + item.port] = s_id
                    if web:
                        if item.severity not in self.ignored_severities:
                            v_id = self.createAndAddVulnWebToService(
                                h_id,
                                s_id,
                                item.name,
                                desc=item.description,
                                website=item.host,
                                severity=item.severity,
                                ref=ref,
                                resolution=item.resolution,
                                external_id=f"OPENVAS-{item.id}",
                                data=item.data,
                                cve=cve,
                                cwe=item.cwe,
                                cvss2=cvss2
                            )
                    elif item.severity not in self.ignored_severities:
                        self.createAndAddVulnToService(
                            h_id,
                            s_id,
                            item.name,
                            desc=item.description,
                            severity=item.severity,
                            ref=ref,
                            resolution=item.resolution,
                            external_id=f"OPENVAS-{item.id}",
                            data=item.data,
                            cve=cve,
                            cwe=item.cwe,
                            cvss2=cvss2
                        )
        del parser

    @staticmethod
    def _isIPV4(ip):
        if len(ip.split(".")) == 4:
            return True
        else:
            return False


def createPlugin(*args, **kwargs):
    return OpenvasPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2016  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
from faraday_plugins.plugins.plugin import PluginBase
import re

import xml.etree.ElementTree as ET

__author__ = 'Ezequiel Tavella'
__copyright__ = 'Copyright (c) 2016, Infobyte LLC'
__credits__ = ['Ezequiel Tavella']
__license__ = ''
__version__ = '1.0.0'
__email__ = "ezequieltbh@infobytesec.com"
__status__ = "Development"


class NdiffXmlParser:
    """
    The objective of this class is to parse an xml file generated by
    the ndiff tool.
    """

    def __init__(self, xmlOutput):
        self.tree = self.parse_xml(xmlOutput)

        if self.tree:
            self.hostDiff = self.getHostsDiffs(self.tree)
        else:
            self.hostDiff = []

    def parse_xml(self, xmlOutput):

        # Open and parse an xml output

        try:
            return ET.fromstring(xmlOutput)
        except SyntaxError as err:
            print(f"SyntaxError: {err}")
            return None

    def getHostsDiffs(self, tree):
        """
        @return hosts A list of HostDiff instances
        """
        for node in tree.findall('scandiff/hostdiff'):
            yield HostDiff(node)


class HostDiff():

    # Abstraction of a Hosts Diff
    # Search for a new host in the second scan and new ports opened or changed
    # of status...
    def __init__(self, hostDiff):

        self.isNewHost = False
        self.hostXml = self.getHostXml(hostDiff)

        self.ip = self.getIp()
        self.ports = self.getPorts()

    def getHostXml(self, hostDiff):

        host = hostDiff.find('host')
        if host is not None:
            return host
        else:
            self.isNewHost = True
            return hostDiff.find('b/host')

    def getIp(self):
        if self.hostXml is None:
            return None
        return self.hostXml.find('address').get('addr')

    def getPorts(self):

        ports = []
        if self.hostXml is None:
            return ports

        if self.isNewHost:

            for port in self.hostXml.find('ports').findall('port'):
                ports.append(
                    [port.get('portid'), port.find('state').get('state')])
            return ports

        else:

            for port in self.hostXml.find('ports').findall('portdiff'):
                if port.find('b/port'):
                    ports.append([port.find('b/port').get('portid'),
                                  port.find('b/port/state').get('state')])
            return ports


class CmdNdiffPlugin(PluginBase):
    """
    This plugin handles ndiff command.
    Add a new vuln INFO if detect a new host or a new port ..
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Ndiff"
        self.name = "ndiff"
        self.plugin_version = "0.0.1"
        self.version = "1.0.0"
        self._command_regex = re.compile(r'^(sudo ndiff|ndiff)\s+.*?')

    def parseOutputString(self, output):
        parser = NdiffXmlParser(output)
        for host in parser.hostDiff:
            if host.ip is None:
                continue
            if host.isNewHost:
                hostId = self.createAndAddHost(host.ip, '')
                description = f'{host.ip} is a NEW host active.\n'
                for port in host.ports:
                    description += f'Port: {port[0]}/{port[1]}\n'
                self.createAndAddVulnToHost(
                    hostId,
                    'New host active',
                    description,
                    ['Ndiff tool'],
                    'INFO'
                )
            else:
                if host.ports == []:
                    continue
                hostId = self.createAndAddHost(host.ip, '')
                description = 'New service/s found.\n'
                for port in host.ports:
                    description += f'Port: {port[0]}/{port[1]}\n'

                self.createAndAddVulnToHost(
                    hostId,
                    'New ports actives',
                    description,
                    ['Ndiff tool'],
                    'INFO'
                )

    def processCommandString(self, username, current_path, command_string):
        super().processCommandString(username, current_path, command_string)
        if command_string.find('--xml') < 0:
            return f"{command_string} --xml "


def createPlugin(*args, **kwargs):
    return CmdNdiffPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2020  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re
import socket
import json
from faraday_plugins.plugins.plugin import PluginJsonFormat
from urllib.parse import urlparse


__author__ = "Blas Moyano"
__copyright__ = "Copyright (c) 2020, Infobyte LLC"
__credits__ = ["Blas Moyano"]
__license__ = ""
__version__ = "0.0.1"
__maintainer__ = "Blas Moyano"
__email__ = "bmoyano@infobytesec.com"
__status__ = "Development"


class WhitesourcePlugin(PluginJsonFormat):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "whitesource"
        self.name = "whitesource"
        self.plugin_version = "0.1"
        self.version = "3.4.5"
        self.json_keys = {"vulnerabilities"}
        self.filter_keys = {"host"}

    def parseOutputString(self, output):
        parser = json.loads(output)
        if parser.get('vulnerabilities'):
            for vulnerability in parser['vulnerabilities']:
                cvss3 = {}
                cvss2 = {}
                if 'project' in vulnerability:
                    application_name = vulnerability.get('project')
                    host_id = self.createAndAddHost(application_name)
                    data = ''
                    for key, value in vulnerability['library'].items():
                        data += f'{key}: {value} \n'
                    refs = []
                    if "scoreMetadataVector" in vulnerability:
                        cvss3["vector_string"] = vulnerability['scoreMetadataVector']
                    if 'topFix' in vulnerability:
                        refs.append(f"URL: {vulnerability['topFix']['url']}")
                        self.createAndAddVulnToHost(host_id,
                                                    name=vulnerability['name'],
                                                    desc=vulnerability['description'],
                                                    data=data,
                                                    resolution=vulnerability['topFix']['fixResolution'],
                                                    ref=refs,
                                                    severity=vulnerability['severity'],
                                                    cve=[vulnerability['name']],
                                                    cvss2=cvss2,
                                                    cvss3=cvss3)
                    else:
                        self.createAndAddVulnToHost(host_id,
                                                    name=vulnerability['name'],
                                                    desc=vulnerability['description'],
                                                    data=data,
                                                    ref=refs,
                                                    severity=vulnerability['severity'],
                                                    cvss2=cvss2,
                                                    cvss3=cvss3)
                elif 'namespace' in vulnerability:
                    host_id = self.createAndAddHost(vulnerability['namespace'])
                    service_id = self.createAndAddServiceToHost(
                        host_id,
                        vulnerability['featurename'],
                        ports=0
                    )
                    self.createAndAddVulnToService(
                        host_id,
                        service_id,
                        name=vulnerability['vulnerability'],
                        desc=vulnerability['description'],
                        ref=[vulnerability['link']],
                        severity=vulnerability['severity'],
                        cve=[vulnerability['vulnerability']]
                    )

                elif 'package' in vulnerability:
                    host_id = self.createAndAddHost(vulnerability['feed_group'])
                    service_id = self.createAndAddServiceToHost(
                        host_id,
                        vulnerability['package'],
                        ports=0
                    )
                    self.createAndAddVulnToService(
                        host_id,
                        service_id,
                        name=f'{vulnerability["vuln"]} {vulnerability["package_name"]}',
                        ref=[vulnerability['url']],
                        severity=vulnerability['severity'],
                        cve=[vulnerability['vuln']]
                    )


def createPlugin(*args, **kwargs):
    return WhitesourcePlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2019  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import json
import re
from urllib.parse import urlparse

from faraday_plugins.plugins.plugin import PluginJsonFormat

__author__ = "Nicolas Rebagliati"
__copyright__ = "Copyright (c) 2019, Infobyte LLC"
__credits__ = ["Nicolas Rebagliati"]
__license__ = ""
__version__ = "0.0.1"
__maintainer__ = "Nicolas Rebagliati"
__email__ = "nrebagliati@infobytesec.com"
__status__ = "Development"


class WPScanJsonParser:

    def __init__(self, json_output, resolve_hostname):
        self.json_data = json.loads(json_output)
        self.resolve_hostname = resolve_hostname

    def parse_url(self, url):
        # Strips protocol and gets hostname from URL.

        url_parse = urlparse(url)
        protocol = url_parse.scheme
        hostname = url_parse.netloc
        port = url_parse.port

        if protocol == 'https':
            port = 443
        elif protocol == 'http':
            if not port:
                port = 80
        address = self.resolve_hostname(hostname)
        return {'protocol': protocol, 'hostname': hostname, 'port': port, 'address': address}


class WPScanPlugin(PluginJsonFormat):
    """ Handle the WPScan tool. Detects the output of the tool
    and adds the information to Faraday.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "wpscan"
        self.name = "WPscan"
        self.plugin_version = "0.2"
        self.version = "3.4.5"
        self.json_keys = [{"target_url", "effective_url", "interesting_findings"},
                          {"target_url", "effective_url", "plugins"}]
        self._command_regex = re.compile(r'^(sudo wpscan|wpscan)\s+.*?')
        self._use_temp_file = True
        self._temp_file_extension = "json"

    def parseOutputString(self, output):
        parser = WPScanJsonParser(output, self.resolve_hostname)
        url_data = parser.parse_url(parser.json_data['target_url'])
        host_id = self.createAndAddHost(url_data['address'], hostnames=[url_data['hostname']])
        service_id = self.createAndAddServiceToHost(
            host_id,
            "WordPress",
            url_data['protocol'],
            ports=[url_data['port']],
            status='open',
            version='',
            description='')
        for user, data in parser.json_data.get('users', {}).items():
            self.createAndAddCredToService(host_id, service_id, user, "")
        main_theme = parser.json_data.get("main_theme", {})
        if main_theme:
            for vuln in main_theme.get("vulnerabilities", []):
                wpvulndb = ",".join(vuln['references'].get('wpvulndb', []))
                self.createAndAddVulnWebToService(host_id, service_id, vuln['title'], ref=vuln['references'].get('url', []),
                                                  severity='unclassified', external_id=wpvulndb)
        for plugin, plugin_data in parser.json_data.get("plugins", {}).items():
            for vuln in plugin_data['vulnerabilities']:
                wpvulndb = ",".join(vuln['references'].get('wpvulndb', []))
                cve = ["CVE-"+ cve for cve in vuln['references'].get('cve')] if vuln['references'].get('cve') else []
                self.createAndAddVulnWebToService(host_id, service_id, f"{plugin}: {vuln['title']}",
                                                  ref=vuln['references'].get('url', []),
                                                  severity='unclassified', external_id=wpvulndb, cve=cve)
        for vuln in parser.json_data.get("interesting_findings", []):
            if vuln['to_s'].startswith('http'):
                vuln_name = f"{vuln['type']}: {vuln['to_s']}"
            else:
                vuln_name = vuln['to_s']
            self.createAndAddVulnWebToService(host_id, service_id, vuln_name, ref=vuln['references'].get('url', []),
                                              severity='unclassified')

    def processCommandString(self, username, current_path, command_string):
        """
        Adds the path to a temporary file parameter to get .json output to the command string that the
        user has set.
        """
        super().processCommandString(username, current_path, command_string)
        command_string += f" --output={self._output_file_path} --format json"
        return command_string


def createPlugin(*args, **kwargs):
    return WPScanPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
from faraday_plugins.plugins.plugin import PluginBase
import re

__author__ = "Federico Fernandez - @q3rv0"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Federico Fernandez"
__email__ = "fede.merlo26@gmail.com"
__status__ = "Development"

class NetdiscoverPlugin(PluginBase):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Netdiscover"
        self.name = "netdiscover"
        self.plugin_version = "0.0.1"
        self.version = "1.0.0"
        self._command_regex = re.compile(r'^(sudo netdiscover|netdiscover)\s+.*?')

    def parseOutputString(self, output):
        #regexp get ip, mac and hostname
        reg = re.findall(r"(([0-9]+\.?){4})\s+(([0-9a-f]+\:?){6})((\s+[0-9]+){2})(.*)", output)

        if output.find('Finished!') != -1 and len(reg) > 0:

            for stdout in reg:
                ip_address = stdout[0]
                mac = stdout[2]
                hostname = stdout[6].strip()
                h_id = self.createAndAddHost(ip_address, hostnames=[hostname])
        return True



def createPlugin(*args, **kwargs):
    return NetdiscoverPlugin(*args, **kwargs)



from typing import List
from urllib.parse import urlparse
import re


def extract_params_from_uri(uri):
    f = re.compile(r"(\w+)=")
    params = re.findall(f, uri)
    return params if params else ''


class Uri:
    def __init__(self, uri):
        self.uri = uri
        self.parsed_url = urlparse(uri)
        self.query = self.parsed_url.query
        self.path = self.parsed_url.path
        self.params = extract_params_from_uri(uri)


class Instance:
    def __init__(self, node):
        self.node = node

    @property
    def uri(self) -> Uri:
        return Uri(self.node.get('uri')) if self.node is not None else ""

    @property
    def method(self) -> str:
        return self.node.get('method') if self.node is not None else ""

    @property
    def param(self) -> str:
        return self.node.get('param') if self.node is not None else ""

    @property
    def attack(self) -> str:
        return self.node.get('attack') if self.node is not None else ""

    @property
    def evidence(self) -> str:
        return self.node.get('evidence') if self.node is not None else ""

    @property
    def request_header(self) -> str:
        return self.node.get('request-header') if self.node is not None else ""

    @property
    def request(self) -> str:
        return self.node.get('request-body') if self.node is not None else ""

    @property
    def response_header(self) -> str:
        return self.node.get('response-header') if self.node is not None else ""

    @property
    def response(self) -> str:
        return self.node.get('response-body') if self.node is not None else ""


class Alert:
    def __init__(self, node):
        self.node = node

    @property
    def plugin_id(self) -> str:
        return self.node.get('pluginid') if self.node is not None else ""

    @property
    def alert_id(self) -> str:
        return self.node.get('alertRef') if self.node is not None else ""

    @property
    def name(self) -> str:
        return self.node.get('name') if self.node is not None else ""

    @property
    def riskcode(self) -> str:
        return self.node.get('riskcode') if self.node is not None else ""

    @property
    def confidence(self) -> str:
        return self.node.get('confidence') if self.node is not None else ""

    @property
    def riskdesc(self) -> str:
        return self.node.get('riskdesc') if self.node is not None else ""

    @property
    def desc(self) -> str:
        return self.node.get('desc') if self.node is not None else ""

    @property
    def instances(self) -> List[Instance]:
        return [Instance(i) for i in self.node.get('instances', [])]

    @property
    def count(self) -> str:
        return self.node.get('count') if self.node is not None else ""

    @property
    def solution(self) -> str:
        return self.node.get('solution') if self.node is not None else ""

    @property
    def otherinfo(self) -> str:
        return self.node.get('otherinfo') if self.node is not None else ""

    @property
    def reference(self) -> str:
        return self.node.get('reference') if self.node is not None else ""

    @property
    def cwe(self) -> str:
        return self.node.get('cweid') if self.node is not None else ""

    @property
    def wasc(self) -> str:
        return self.node.get('wascid') if self.node is not None else ""

    @property
    def sourceid(self) -> str:
        return self.node.get("sourceid") if self.node is not None else ""


class Site:
    def __init__(self, node):
        self.node = node

    @property
    def host(self) -> str:
        return self.node.get('@host') if self.node is not None else ""

    @property
    def name(self) -> str:
        return self.node.get('@name') if self.node is not None else ""

    @property
    def port(self) -> str:
        return self.node.get('@port') if self.node is not None else ""

    @property
    def ssl(self) -> str:
        return self.node.get('@ssl') if self.node is not None else ""

    @property
    def alerts(self) -> List[Alert]:
        return [Alert(i) for i in self.node.get('alerts', [])]


class ZapJsonParser:
    def __init__(self, node):
        self.node = node

    @property
    def sites(self) -> List[Site]:
        return [Site(i) for i in self.node.get('site', [])]



"""
Updated by Mike Zhong, 25 Oct 2017.

Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import re

__author__ = "Andres Tarantini"
__copyright__ = "Copyright (c) 2015 Andres Tarantini"
__credits__ = ["Andres Tarantini"]
__license__ = "MIT"
__version__ = "0.0.1"
__maintainer__ = "Andres Tarantini"
__email__ = "atarantini@gmail.com"
__status__ = "Development"

from faraday_plugins.plugins.plugin import PluginBase


class DigPlugin(PluginBase):
    """
    Handle DiG (http://linux.die.net/man/1/dig) output
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "dig"
        self.name = "DiG"
        self.plugin_version = "0.0.1"
        self.version = "9.9.5-3"
        self._command_regex = re.compile(r'^(dig)\s+.*?')

    def parseOutputString(self, output):
        # Ignore all lines that start with ";"
        parsed_output = [line for line in output.splitlines() if line and line[
            0] != ";"]
        if not parsed_output:
            return True

        # Parse results
        results = []
        answer_section_columns = ["domain",
                                  "ttl", "class", "type", "data"]
        for line in parsed_output:
            line_split = line.split() # the first 4 elements are domain, ttl, class, type; everything else data
            results.append(dict(zip(answer_section_columns, line_split[:4] + [line_split[4:]] )))

        # Create hosts is results information is relevant
        try:
            for result in results:
                relevant_types = ["A", "AAAA", "MX", "NS", "SOA", "TXT"]
                # TODO implement more types from https://en.wikipedia.org/wiki/List_of_DNS_record_types

                if result.get("type") in relevant_types:

                    # get domain
                    domain = result.get("domain")


                    # get IP address (special if type "A")
                    if result.get("type") == "A": # A = IPv4 address from dig
                        ip_address = result.get("data")[0]
                    else:                           # if not, from socket
                        ip_address = self.resolve_hostname(domain)

                    # Create host
                    host_id = self.createAndAddHost(ip_address, hostnames=[domain])


                    # all other TYPES that aren't 'A' and 'AAAA' are dealt here:
                    if result.get("type") == "MX": # Mail exchange record
                        mx_priority = result.get("data")[0]
                        mx_record = result.get("data")[1]

                        service_id = self.createAndAddServiceToHost(
                            host_id=host_id,
                            name=mx_record,
                            protocol="SMTP",
                            ports=[25],
                            description="E-mail Server")

                        text = "Priority: " + mx_priority
                        self.createAndAddNoteToService(
                            host_id=host_id,
                            service_id=service_id,
                            name="priority",
                            text=text.encode('ascii', 'ignore'))

                    elif result.get("type") == "NS": # Name server record
                        ns_record = result.get("data")[0]
                        self.createAndAddServiceToHost(
                            name=ns_record,
                            protocol="DNS",
                            ports=[53],
                            description="DNS Server")

                    elif result.get("type") == "SOA": # Start of Authority Record
                        ns_record = result.get("data")[0] # primary namer server
                        responsible_party = result.get("data")[1] # responsible party of domain
                        timestamp = result.get("data")[2]
                        refresh_zone_time = result.get("data")[3]
                        retry_refresh_time = result.get("data")[4]
                        upper_limit_time = result.get("data")[5]
                        negative_result_ttl = result.get("data")[6]

                        service_id = self.createAndAddServiceToHost(
                            host_id=host_id,
                            name=ns_record,
                            protocol="DNS",
                            ports=[53],
                            description="Authority Record")

                        text = (
                            "Responsible Party: " + responsible_party +
                            "\nTimestep: " + timestamp +
                            "\nTime before zone refresh (sec): " + refresh_zone_time +
                            "\nTime before retry refresh (sec): " + retry_refresh_time +
                            "\nUpper Limit before Zone is no longer authoritive (sec): " + upper_limit_time +
                            "\nNegative Result TTL: " + negative_result_ttl)

                        self.createAndAddNoteToService(
                            host_id=host_id,
                            service_id=service_id,
                            name="priority",
                            text=text.encode('ascii', 'ignore'))

                    elif result.get("type") == "TXT": # TXT record
                        text = " ".join(result.get("data")[:])
                        self.createAndAddNoteToHost(
                            host_id=host_id,
                            name="TXT Information",
                            text=text.encode('ascii', 'ignore'))

        except Exception as ex:
            print("some part of the dig plug-in caused an error! Please check repo/dig/plugin.py")
            return False

        return True


def createPlugin(*args, **kwargs):
    return DigPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re

from faraday_plugins.plugins.plugin import PluginBase

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class DnswalkParser:
    """
    The objective of this class is to parse an xml file generated
    by the dnswalk tool.

    TODO: Handle errors.
    TODO: Test dnswalk output version. Handle what happens if the parser
    doesn't support it.
    TODO: Test cases.

    @param dnswalk_filepath A proper simple report generated by dnswalk
    """

    def __init__(self, output, resolve_hostName):

        lists = output.split("\n")
        self.items = []
        self.resolve_hostName = resolve_hostName

        for line in lists:
            mregex = re.search(r"WARN: ([\w\.]+) ([\w]+) ([\w\.]+):", line)
            if mregex is not None:
                item = {
                    'host': mregex.group(1),
                    'ip': mregex.group(3),
                    'type': mregex.group(2)}

                self.items.append(item)

            mregex = re.search(
                r"Getting zone transfer of ([\w\.]+) from ([\w\.]+)\.\.\.done\.",
                line)

            if mregex is not None:
                ip = self.resolve_hostname(mregex.group(2))
                item = {
                    'host': mregex.group(1),
                    'ip': ip,
                    'type': 'info'}
                self.items.append(item)


class DnswalkPlugin(PluginBase):
    """
    Example plugin to parse dnswalk output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Dnswalk"
        self.name = "Dnswalk XML Output Plugin"
        self.plugin_version = "0.0.1"
        self.version = "2.0.2"
        self.options = None
        self._current_output = None
        self._command_regex = re.compile(
            r'^(sudo dnswalk|dnswalk|\.\/dnswalk)\s+.*?')

    def canParseCommandString(self, current_input):
        if self._command_regex.match(current_input.strip()):
            self.command = self.get_command(current_input)
            return True
        else:
            return False

    def parseOutputString(self, output):
        """
        output is the shell output of command Dnswalk.
        """
        parser = DnswalkParser(output, self.resolve_hostname)

        for item in parser.items:
            if item['type'] == "A":
                h_id = self.createAndAddHost(item['ip'], hostnames=[item['host']])
            elif item['type'] == "info":
                h_id = self.createAndAddHost(item['ip'], hostnames=[item['host']])
                s_id = self.createAndAddServiceToHost(
                    h_id,
                    "domain",
                    "tcp",
                    ports=['53'])
                self.createAndAddVulnToService(
                    h_id,
                    s_id,
                    "Zone transfer",
                    desc="A Dns server allows unrestricted zone transfers",
                    ref=["CVE-1999-0532"])

        return True


def createPlugin(*args, **kwargs):
    return DnswalkPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import re
from faraday_plugins.plugins.plugin import PluginBase


__author__ = "Francisco Amato"
__copyright__ = "Copyright 2013, Faraday Project"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class MedusaParser:
    """
    The objective of this class is to parse an xml file generated by the medusa tool.

    @param medusa_filepath A proper simple report generated by medusa
    """

    def __init__(self, xml_output, resolve_hostname):
        self.resolve_hostname = resolve_hostname
        self.srv = {
            'ftp': '21',  'http': '80', 'imap': '143', 'mssql': '1433', 'mysql': '3306',
            'ncp': '524', 'nntp': '119', 'pcanywhere': '5631', 'pop3': '110', 'postgres': '5432',
            'rexec': '512', 'rlogin': '513', 'rsh': '514', 'smbnt': 'smbnt', 'smtp': '25',
            'smtp-vrfy': 'smtp-vrfy', 'snmp': '161', 'ssh': '22', 'svn': '3690',
            'telnet': '23', 'vmauthd': 'vmauthd', 'vnc': '5900', 'web-form': 'web-form',
            'wrapper': 'wrapper'
        }

        lines = xml_output.splitlines()
        self.items = []

        for line in lines:

            reg = re.search(r"ACCOUNT FOUND: \[([^$]+)\] Host: ([^$]+) User: ([^$]+) Password: ([^$]+) \[SUCCESS\]", line)
            if reg:

                item = {
                    'service': reg.group(1),
                    'host': reg.group(2),
                    'user': reg.group(3),
                    'pass': reg.group(4)
                }
                item['ip'] = resolve_hostname(item['host'])
                item['port'] = self.srv[item['service']]
                self.items.append(item)


class MedusaPlugin(PluginBase):
    """
    Example plugin to parse medusa output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Medusa"
        self.name = "Medusa Output Plugin"
        self.plugin_version = "0.0.1"
        self.version = "2.1.1"
        self.options = None
        self._command_regex = re.compile(r'^(sudo medusa|sudo \.\/medusa|medusa|\.\/medusa)\s+.*?')
        self.host = None
        self.port = ""
        self._use_temp_file = True
        self._temp_file_extension = "txt"
        self.xml_arg_re = re.compile(r"^.*(-O\s*[^\s]+).*$")

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it from
        the xml where it expects it to be present.

        NOTE: if 'debug' is true then it is being run from a test case and the
        output being sent is valid.
        """
        parser = MedusaParser(output, resolve_hostname=self.resolve_hostname)

        for item in parser.items:

            h_id = self.createAndAddHost(item['ip'], hostnames=[item['host']])

            port = self.port if self.port else item['port']

            s_id = self.createAndAddServiceToHost(
                h_id,
                item['service'],
                ports=[port],
                protocol="tcp",
                status="open")

            self.createAndAddCredToService(
                h_id,
                s_id,
                item['user'],
                item['pass'])

            self.createAndAddVulnToService(h_id,
                s_id,
                "Weak Credentials",
                f"[medusa found the following credentials]\nuser:{item['user']}\npass:{item['pass']}",
                severity="high")

        del parser

    def processCommandString(self, username, current_path, command_string):
        super().processCommandString(username, current_path, command_string)
        self.port = ""
        arg_match = self.xml_arg_re.match(command_string)

        mreg = re.search(r"\-n( |)([\d]+)", command_string)
        if mreg:
            self.port = mreg.group(2)

        if arg_match is None:
            return re.sub(r"(^.*?medusa?)", r"\1 -O %s" % self._output_file_path, command_string)
        else:
            return re.sub(arg_match.group(1), r"-O %s" % self._output_file_path, command_string)

    def _isIPV4(self, ip):
        if len(ip.split(".")) == 4:
            return True
        else:
            return False




def createPlugin(*args, **kwargs):
    return MedusaPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import re
from os import path
from urllib.parse import urlparse

__author__ = "Andres Tarantini"
__copyright__ = "Copyright (c) 2015 Andres Tarantini"
__credits__ = ["Andres Tarantini"]
__license__ = "MIT"
__version__ = "0.0.1"
__maintainer__ = "Andres Tarantini"
__email__ = "atarantini@gmail.com"
__status__ = "Development"

from faraday_plugins.plugins.plugin import PluginBase


class PeepingTomPlugin(PluginBase):
    """
    Handle PeepingTom (https://bitbucket.org/LaNMaSteR53/peepingtom) output
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "peepingtom"
        self.name = "PeepingTom"
        self.plugin_version = "0.0.1"
        self.version = "02.19.15"
        self._command_regex = re.compile(
            r'^(python peepingtom.py|\./peepingtom.py)\s+.*?')
        self._path = None

    def parseOutputString(self, output):
        # Find data path
        data_path_search = re.search(r"in '(.*)\/'", output)
        print(data_path_search)
        if not data_path_search:
            # No data path found
            return True

        # Parse "peepingtom.html" report and extract results
        data_path = data_path_search.groups()[0]
        html = open(path.join(self._path, data_path, "peepingtom.html")).read()
        for url in re.findall(r'href=[\'"]?([^\'" >]+)', html):
            if "://" in url:
                url_parsed = urlparse(url)
                address = self.resolve_hostname(url_parsed.netloc)
                host = self.createAndAddHost(address)
                service = self.createAndAddServiceToHost(host, "http", protocol="tcp", ports=[80])
                self.createAndAddNoteToService(
                    host,
                    service,
                    'screenshot',
                    path.join(
                        self._path,
                        data_path_search.groups()[0],
                        "{}.png".format(url.replace(
                            "://", "").replace("/", "").replace(".", ""))
                    )
                )

        return True

    def processCommandString(self, username, current_path, command_string):
        super().processCommandString(username, current_path, command_string)
        self._path = current_path


def createPlugin(*args, **kwargs):
    return PeepingTomPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""

import json
import re
from faraday_plugins.plugins.plugin import PluginJsonFormat

__author__ = "Joachim Bauernberger"
__license__ = "MIT"
__version__ = "1.0.0"
__maintainer__ = "Joachim Bauernberger"
__email__ = "joachim.bauernberger@protonmail.com"
__status__ = "Development"


class GrypePlugin(PluginJsonFormat):
    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = 'grype'
        self.name = 'Grype JSON Plugin'
        self.plugin_version = '1.0.0'
        self._command_regex = re.compile(r'^grype\s+.*')
        self._use_temp_file = True
        self._temp_file_extension = "json"
        self.json_keys = [{"source", "matches", "descriptor"}, {"matches", "image"}]

    def parseOutputString(self, output, debug=True):
        grype_json = json.loads(output)
        if "userInput" in grype_json.get("source", {"target": ""}).get("target"):
            name = grype_json["source"]["target"]["userInput"]
            host_type = grype_json['source']['type']
        elif "tags" in grype_json.get("image", {}):
            name = " ".join(grype_json["image"]["tags"])
            host_type = "Docker Image"
        else:
            name = grype_json["source"]["target"]
            host_type = grype_json['source']['type']
        host_id = self.createAndAddHost(name, description=f"Type: {host_type}")
        for match in grype_json['matches']:
            name = match.get('vulnerability').get('id')
            cve = name
            references = []
            if match.get("relatedVulnerabilities"):
                description = match["relatedVulnerabilities"][0].get('description')
                references.append(match["relatedVulnerabilities"][0]["dataSource"])
                related_vuln = match["relatedVulnerabilities"][0]
                severity = related_vuln["severity"].lower().replace("negligible", "info")
                if related_vuln.get("links"):
                    for url in related_vuln["links"]:
                        references.append(url)
                else:
                    for url in related_vuln["urls"]:
                        references.append(url)
            else:
                description = match.get('vulnerability').get('description', "Issues provided no description")
                severity = match.get('vulnerability').get('severity').lower().replace("negligible", "info")
                if match.get('vulnerability').get("links"):
                    for url in match.get('vulnerability')["links"]:
                        references.append(url)
                else:
                    for url in match.get('vulnerability')["urls"]:
                        references.append(url)
            if not match['artifact'].get('metadata'):
                data = f"Artifact: {match['artifact']['name']}" \
                       f"Version: {match['artifact']['version']} " \
                       f"Type: {match['artifact']['type']}"
            else:
                if "Source" in match['artifact']['metadata']:
                    data = f"Artifact: {match['artifact']['name']} [{match['artifact']['metadata']['Source']}] " \
                           f"Version: {match['artifact']['version']} " \
                           f"Type: {match['artifact']['type']}"
                elif "VirtualPath" in match['artifact']['metadata']:
                    data = f"Artifact: {match['artifact']['name']} [{match['artifact']['metadata']['VirtualPath']}] " \
                           f"Version: {match['artifact']['version']} " \
                           f"Type: {match['artifact']['type']}"
                else:
                    data = f"Artifact: {match['artifact']['name']}" \
                           f"Version: {match['artifact']['version']} " \
                           f"Type: {match['artifact']['type']}"
            self.createAndAddVulnToHost(host_id,
                                        name=name,
                                        desc=description,
                                        ref=references,
                                        severity=severity,
                                        data=data,
                                        cve=cve)

    def processCommandString(self, username, current_path, command_string):
        super().processCommandString(username, current_path, command_string)
        command_string += f" -o json --file {self._output_file_path}"
        return command_string


def createPlugin(*args, **kwargs):
    return GrypePlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re
import os
import json
import random
import shutil
import tempfile

from faraday_plugins.plugins.plugin import PluginBase


__author__ = "Nicolas Rodriguez"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Nicolas Rodriguez"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class SkipfishParser:
    """
    The objective of this class is to parse an xml file generated by
    the skipfish tool.

    TODO: Handle errors.
    TODO: Test skipfish output version. Handle what happens if the parser
    doesn't support it.
    TODO: Test cases.

    @param skipfish_filepath A proper xml generated by skipfish
    """

    def __init__(self, skipfish_filepath):
        self.filepath = skipfish_filepath

        tmp = open(skipfish_filepath + "/samples.js").read()
        data = self.extract_data(
            tmp,
            "var issue_samples =", "];",
            lambda x: x.replace("'", '"'),
            False,
            False)
        # Escape characters not allowed in JSON, repr fix this with double Escape
        # Also remove \n character and space for have a valid JSON.
        issues = json.loads(repr(data[1]).replace("\\n"," ").replace("'","") + "]")

        tmp = open(skipfish_filepath + "/index.html").read()
        err_msg = json.loads(
            self.extract_data(
                tmp,
                "var issue_desc=",
                "};",
                lambda x: self.convert_quotes(x, "'", '"'),
                False,
                False)
            [1] + "}")

        self.err_msg = err_msg
        self.issues = issues

    def convert_quotes(self, text, quote="'", inside='"'):
        start = 0
        while True:
            pos = text.find(quote, start)

            if pos == -1:
                break

            ss = text[:pos - 1]
            quotes = len(ss) - len(ss.replace(inside, ""))

            if quotes % 2 == 0:
                text = text[:pos - 1] + "\\" + quote + text[pos + 1:]

            start = pos + 1
        return text

    def extract_data(self, samples, start_tag, end_tag, fn=lambda x: x, include_start_tag=True, include_end_tag=True):
        start = samples.find(start_tag)

        if start == -1:
            return (-1, None)

        end = samples.find(end_tag, start + 1)

        if end == -1:
            return (-2, None)

        data = samples[start:end + len(end_tag)]
        data = fn(data)

        if not include_start_tag:
            data = data[len(start_tag) + 1:]

        if not include_end_tag:
            data = data[:-1 * len(end_tag)]

        return (0, data)


class SkipfishPlugin(PluginBase):
    """
    Example plugin to parse skipfish output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Skipfish"
        self.name = "Skipfish Output Plugin"
        self.plugin_version = "0.0.2"
        self.version = "2.1.5"
        self.options = None
        self.parent = None
        self._command_regex = re.compile(
            r'^(sudo skipfish|skipfish|sudo skipfish\.pl|skipfish\.pl|perl skipfish\.pl|\.\/skipfish\.pl|\.\/skipfish)\s+.*?')

    def _parse_filename(self, filename):
        self.parseOutputString(filename)
        if self._delete_temp_file:
            try:
                if os.path.isfile(filename):
                    os.remove(filename)
                elif os.path.isdir(filename):
                    shutil.rmtree(filename)
            except Exception as e:
                self.logger.error("Error on delete file: (%s) [%s]", filename, e)

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it
        from the xml where it expects it to be present.

        NOTE: if 'debug' is true then it is being run from a test case and the
        output being sent is valid.
        """

        if not os.path.isdir(self._output_file_path):
            return False

        p = SkipfishParser(self._output_file_path)

        hostc = {}
        for issue in p.issues:
            for sample in issue["samples"]:
                if not sample["url"] in hostc:
                    reg = re.search(
                        "(http|https|ftp)\\://([a-zA-Z0-9\\.\\-]+(\\:[a-zA-Z0-9\\.&amp;%\\$\\-]+)*@)*((25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9])\\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[0-9])|localhost|([a-zA-Z0-9\\-]+\\.)*[a-zA-Z0-9\\-]+\\.(com|edu|gov|int|mil|net|org|biz|arpa|info|name|pro|aero|coop|museum|[a-zA-Z]{2}))[\\:]*([0-9]+)*([/]*($|[a-zA-Z0-9\\.\\,\\?\'\\\\+&amp;%\\$#\\=~_\\-]+)).*?$", sample["url"])

                    protocol = reg.group(1)
                    host = reg.group(4)
                    if reg.group(11) is not None:
                        port = reg.group(11)
                    else:
                        port = 443 if protocol == "https" else 80

                    ip = self.resolve_hostname(host)

                    h_id = self.createAndAddHost(ip, hostnames=[host])
                    s_id = self.createAndAddServiceToHost(h_id, "http", "tcp", ports=[port], status="open")

                    hostc[sample["url"]] = {
                        'h_id': h_id,
                        'ip': ip,
                        'port': port,
                        'host': host,
                        'protocol': protocol,
                        's_id': s_id}

                d = hostc[sample["url"]]
                self.createAndAddVulnWebToService(
                    d['h_id'],
                    d['s_id'],
                    name=p.err_msg[str(issue["type"])],
                    desc="Extra: " + sample["extra"],
                    website=d['host'],
                    path=sample["url"],
                    severity=issue["severity"])


    xml_arg_re = re.compile(r"^.*(-o\s*[^\s]+).*$")

    def processCommandString(self, username, current_path, command_string):
        """
        Adds the -o parameter to get report of the command string that the
        user has set.
        """
        super().processCommandString(username, current_path, command_string)
        arg_match = self.xml_arg_re.match(command_string)
        self._output_file_path = os.path.join(tempfile.gettempdir(),
                                              "faraday_plugin_skipfish_%d" % random.randint(1, 999999))  # nosec
        self._delete_temp_file = True
        if arg_match is None:
            return re.sub(r"(^.*?skipfish)", r"\1 -o %s" % self._output_file_path, command_string, 1)
        else:
            return re.sub(arg_match.group(1), r"-o %s" % self._output_file_path, command_string, 1)




def createPlugin(*args, **kwargs):
    return SkipfishPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import re
import os

from faraday_plugins.plugins.plugin import PluginBase

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class GoohostParser:
    """
    The objective of this class is to parse an xml file generated by the goohost tool.

    TODO: Handle errors.
    TODO: Test goohost output version. Handle what happens if the parser doesn't support it.
    TODO: Test cases.

    @param goohost_scantype You could select scan type ip, mail or host
    """

    def __init__(self, output, goohost_scantype, resolve_hostname):

        self.items = []
        self.resolve_hostname = resolve_hostname
        lines = list(filter(None, output.split('\n')))
        for line in lines:
            if goohost_scantype == 'ip':
                data = line.split()
                item = {'host': data[0], 'ip': data[1]}
                self.add_host_info_to_items(item['ip'], item['host'])
            elif goohost_scantype == 'host':
                data = line.strip()
                item = {'host': data, 'ip': self.resolve_hostname(data)}
                self.add_host_info_to_items(item['ip'], item['host'])
            else:
                item = {'data': line}

    def add_host_info_to_items(self, ip_address, hostname):
        data = {}
        exists = False
        for item in self.items:
            if ip_address in item['ip']:
                item['hosts'].append(hostname)
                exists = True

        if not exists:
            data['ip'] = ip_address
            data['hosts'] = [hostname]
            self.items.append(data)


class GoohostPlugin(PluginBase):
    """
    Example plugin to parse goohost output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Goohost"
        self.name = "Goohost XML Output Plugin"
        self.plugin_version = "0.0.1"
        self.version = "v.0.0.1"
        self.options = None
        self._command_regex = re.compile(r'^(sudo goohost\.sh|goohost\.sh|sh goohost\.sh|\.\/goohost\.sh)\s+.*?')
        self.host = None
        self._command_string = None

    def parseOutputString(self, output):
        """
        This method will check if the import was made through the console or by importing a Goohost report.

        Import from Console:The method will take the path of the report generated by Goohost from the output the shell sends and will read
        the information from the txt where it expects it to be present.

        Import from Report: The method receives the output of the txt report as parameter.

        self.scantype defines the method used to generate the Goohost report

        """
        scantype = self.define_scantype_by_output(output)
        parser = GoohostParser(output, scantype, self.resolve_hostname)
        if scantype == 'host' or scantype == 'ip':
            for item in parser.items:
                h_id = self.createAndAddHost(item['ip'], hostnames=item['hosts'])
        del parser

    def define_scantype_by_command(self, command):
        method_regex = re.compile(r'-m (mail|host|ip)')
        method = method_regex.search(command)
        if method:
            return method.group(1)
        return 'host'

    def define_scantype_by_output(self, output):
        lines = output.split('\n')
        line = lines[0].split(' ')
        if len(line) == 1:
            return 'host'
        elif len(line) == 2:
            return 'ip'

    def get_report_path_from_output(self, command_output):
        report_name = re.search(r"Results saved in file (\S+)", command_output)
        if not report_name:
            return False
        else:
            self._output_file_path = os.path.join(self._current_path, report_name.group(1))
            if not os.path.exists(self._output_file_path):
                return False
            else:
                self._delete_temp_file = True

    def processOutput(self, command_output):
        self.get_report_path_from_output(command_output)
        if self.has_custom_output():
            self._parse_filename(self.get_custom_file_path())
        else:
            self.parseOutputString(command_output)


def createPlugin(*args, **kwargs):
    return GoohostPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re
import xml.etree.ElementTree as ET

from faraday_plugins.plugins.plugin import PluginBase

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class DnsenumXmlParser:
    """
    The objective of this class is to parse an xml file generated by the dnsenum tool.

    TODO: Handle errors.
    TODO: Test dnsenum output version. Handle what happens if the parser doesn't support it.
    TODO: Test cases.

    @param dnsenum_xml_filepath A proper xml generated by dnsenum
    """

    def __init__(self, xml_output):
        tree = self.parse_xml(xml_output)

        if tree:
            self.items = [data for data in self.get_items(tree)]
        else:
            self.items = []

    def parse_xml(self, xml_output):
        """
        Open and parse an xml file.

        TODO: Write custom parser to just read the nodes that we need instead of
        reading the whole file.

        @return xml_tree An xml tree instance. None if error.
        """
        try:
            tree = ET.fromstring(xml_output)
        except SyntaxError as err:
            print(f"SyntaxError: {err}. {xml_output}")
            return None

        return tree

    def get_items(self, tree):
        """
        @return items A list of Host instances
        """

        node = tree.findall('testdata')[0]
        for hostnode in node.findall('host'):
            yield Item(hostnode)


def get_attrib_from_subnode(xml_node, subnode_xpath_expr, attrib_name):
    """
    Finds a subnode in the item node and the retrieves a value from it

    @return An attribute value
    """

    node = xml_node.find(subnode_xpath_expr)

    if node is not None:
        return node.get(attrib_name)

    return None


class Item:
    """
    An abstract representation of a Item

    TODO: Consider evaluating the attributes lazily
    TODO: Write what's expected to be present in the nodes
    TODO: Refactor both Host and the Port clases?

    @param item_node A item_node taken from an dnsenum xml tree
    """

    def __init__(self, item_node):
        self.node = item_node

        self.hostname = self.get_text_from_subnode('hostname')
        self.ip = self.node.text

    def do_clean(self, value):
        myreturn = ""
        if value is not None:
            myreturn = re.sub("\n", "", value)
        return myreturn

    def get_text_from_subnode(self, subnode_xpath_expr):
        """
        Finds a subnode in the host node and the retrieves a value from it.

        @return An attribute value
        """
        sub_node = self.node.find(subnode_xpath_expr)
        if sub_node is not None:
            return sub_node.text

        return None


class DnsenumPlugin(PluginBase):
    """
    Example plugin to parse dnsenum output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Dnsenum"
        self.name = "Dnsenum XML Output Plugin"
        self.plugin_version = "0.0.1"
        self.version = "1.2.2"
        self.options = None
        self._current_output = None
        self._use_temp_file = True
        self._temp_file_extension = "txt"
        self._command_regex = re.compile(
            r'^(sudo dnsenum|dnsenum|sudo dnsenum\.pl|dnsenum\.pl|perl dnsenum\.pl|\.\/dnsenum\.pl)\s+.*?')
        self.xml_arg_re = re.compile(r"^.*(-o\s*[^\s]+).*$")

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it from
        the xml where it expects it to be present.

        NOTE: if 'debug' is true then it is being run from a test case and the
        output being sent is valid.
        """

        parser = DnsenumXmlParser(output)

        for item in parser.items:
            self.createAndAddHost(item.ip, hostnames=[item.hostname])

        del parser

    def processCommandString(self, username, current_path, command_string):
        """
        Adds the -oX parameter to get xml output to the command string that the
        user has set.
        """
        super().processCommandString(username, current_path, command_string)
        arg_match = self.xml_arg_re.match(command_string)

        if arg_match is None:
            return re.sub(r"(^.*?dnsenum(\.pl)?)", r"\1 -o %s" % self._output_file_path, command_string)
        else:
            return re.sub(arg_match.group(1), r"-o %s" % self._output_file_path, command_string)


def createPlugin(*args, **kwargs):
    return DnsenumPlugin(*args, **kwargs)



from typing import List


class Attachment:
    def __init__(self, node):
        self.node = node

    @property
    def name_attr(self):
        return self.node.get("name")

    @property
    def type_attr(self):
        return self.node.get("type")

    @property
    def text(self):
        return self.node.text


class ReportItem:
    def __init__(self, node):
        self.node = node

    @property
    def port_attr(self):
        return self.node.get("port")

    @property
    def svc_name_attr(self):
        return self.node.get("svc_name")

    @property
    def protocol_attr(self):
        return self.node.get("protocol")

    @property
    def severity_attr(self):
        return self.node.get("severity")

    @property
    def plugin_id_attr(self):
        plugin_id = self.node.get("pluginID")
        if plugin_id:
            plugin_id = f'NESSUS-{plugin_id}'
        return plugin_id

    @property
    def plugin_name_attr(self):
        return self.node.get("pluginName")

    @property
    def plugin_family_attr(self):
        return self.node.get("pluginFamily")

    @property
    def agent(self):
        return self.node.findtext("agent")

    @property
    def description(self):
        return self.node.findtext("description", "Not Description")

    @property
    def fname(self):
        return self.node.findtext("fname")

    @property
    def plugin_modification_date(self):
        return self.node.findtext("plugin_modification_date")

    @property
    def plugin_name(self):

        plugin_name = self.node.findtext("plugin_name")
        if not plugin_name:
            plugin_name = self.plugin_name_attr
        return plugin_name

    @property
    def plugin_publication_date(self):
        return self.node.findtext("plugin_publication_date")

    @property
    def plugin_type(self):
        return self.node.findtext("plugin_type")

    @property
    def risk_factor(self):
        risk_factor = self.node.findtext("risk_factor")
        if risk_factor == 'None' or risk_factor is None:
            risk_factor = self.severity_attr  # I checked several external id and most of them were info
        return risk_factor

    @property
    def script_version(self):
        return self.node.findtext("script_version")

    @property
    def see_also(self):
        return self.node.findtext("see_also")

    @property
    def solution(self):
        return self.node.findtext("solution", '')

    @property
    def synopsis(self):
        return self.node.findtext("synopsis")

    @property
    def plugin_output(self):
        return self.node.findtext("plugin_output", "")

    @property
    def always_run(self):
        return self.node.findtext("always_run")

    @property
    def asset_inventory(self):
        return self.node.findtext("asset_inventory")

    @property
    def canvas_package(self):
        return self.node.findtext("canvas_package")

    @property
    def cvss3_base_score(self):
        return self.node.findtext("cvss3_base_score")

    @property
    def cvss3_temporal_score(self):
        return self.node.findtext("cvss3_temporal_score")

    @property
    def cpe(self):
        return self.node.findtext("cpe")

    @property
    def cvss3_temporal_vector(self):
        return self.node.findtext("cvss3_temporal_vector")

    @property
    def cvss3_vector(self):
        return self.node.findtext("cvss3_vector")

    @property
    def cvss2_base_score(self):
        return self.node.findtext("cvss_base_score")

    @property
    def cvss_score_rationale(self):
        return self.node.findtext("cvss_score_rationale")

    @property
    def cvss_score_source(self):
        return self.node.findtext("cvss_score_source")

    @property
    def cvss_temporal_score(self):
        return self.node.findtext("cvss_temporal_score")

    @property
    def cvss_temporal_vector(self):
        return self.node.findtext("cvss_temporal_vector")

    @property
    def cvss_vector(self):
        cvss_vector = self.node.findtext("cvss_vector")
        if cvss_vector:
            cvss_vector = cvss_vector.replace("CVSS2#", "")
        return cvss_vector

    @property
    def exploit_available(self):
        exploit_avalible = self.node.findtext("exploit_available", "")
        if exploit_avalible:
            exploit_avalible = f"Exploit available: {exploit_avalible.capitalize()}\n"
        return exploit_avalible

    @property
    def exploit_framework_canvas(self):
        return self.node.findtext("exploit_framework_canvas")

    @property
    def exploit_framework_core(self):
        return self.node.findtext("exploit_framework_core")

    @property
    def exploit_framework_d2_elliot(self):
        return self.node.findtext("exploit_framework_d2_elliot")

    @property
    def exploit_framework_metasploit(self):
        return self.node.findtext("exploit_framework_metasploit")

    @property
    def exploitability_ease(self):
        return self.node.findtext("exploitability_ease")

    @property
    def exploited_by_malware(self):
        return self.node.findtext("exploited_by_malware")

    @property
    def exploited_by_nessus(self):
        return self.node.findtext("exploited_by_nessus")

    @property
    def hardware_inventory(self):
        return self.node.findtext("hardware_inventory")

    @property
    def iava(self):
        return self.node.findtext("iava")

    @property
    def iavb(self):
        return self.node.findtext("iavb")

    @property
    def iavt(self):
        return self.node.findtext("iavt")

    @property
    def in_the_news(self):
        return self.node.findtext("in_the_news")

    @property
    def metasploit_name(self):
        return self.node.findtext("metasploit_name")

    @property
    def os_identification(self):
        return self.node.findtext("os_identification")

    @property
    def owasp(self):
        return self.node.findtext("owasp")

    @property
    def patch_publication_date(self):
        return self.node.findtext("patch_publication_date")


    @property
    def stig_severity(self):
        return self.node.findtext("stig_severity")

    @property
    def d2_elliot_name(self):
        return self.node.findtext("d2_elliot_name")

    @property
    def unsupported_by_vendor(self):
        return self.node.findtext("unsupported_by_vendor")

    @property
    def vuln_publication_date(self):
        return self.node.findtext("vuln_publication_date")

    @property
    def msft(self):
        return self.node.findtext("msft")


    @property
    def cert(self) -> list:
        return self.node.findall("cert")

    @property
    def bid(self) -> list:
        return self.node.findall("bid")

    @property
    def cve(self) -> list:
        return [i.text for i in self.node.findall("cve")]

    @property
    def cwe(self) -> list:
        return ["CWE-"+i.text for i in self.node.findall("cwe")]

    @property
    def edb_id(self) -> list:
        return self.node.findall("edb-id")

    @property
    def mskb(self) -> list:
        return self.node.findall("mskb")

    @property
    def xref(self) -> str:
        return self.node.findtext("xref")

    @property
    def attachment(self) -> Attachment:
        attachment = self.node.find("attachment")
        return Attachment(attachment) if attachment else None

    def get_data(self):
        item_tags = {}
        for i in self.node:
            item_tags.setdefault(i.tag, i.text)
        return item_tags


class Tag:
    def __init__(self, node):
        self.node = node

    @property
    def name_attr(self) -> str:
        return self.node.get("name")

    @property
    def text(self) -> str:
        return self.node.text


class HostProperties:
    def __init__(self, node):
        self.node = node

    @property
    def tag(self) -> list:
        return [Tag(i) for i in self.node.findall('tag')]

    @property
    def host_end(self) -> str:
        _dict = self.dict_tags
        return _dict.get("HOST_END")

    @property
    def mac_address(self) -> str:
        _dict = self.dict_tags
        return _dict.get("mac-address", "")

    @property
    def operating_system(self) -> str:
        _dict = self.dict_tags
        return _dict.get("operating-system", None)

    @property
    def host_ip(self) -> str:
        _dict = self.dict_tags

        return _dict.get("host-ip", None)

    @property
    def host_fqdn(self) -> str:
        _dict = self.dict_tags
        return _dict.get("host-fqdn", None)

    @property
    def host_rdns(self) -> str:
        _dict = self.dict_tags
        return _dict.get("host-rdns", None)

    @property
    def dict_tags(self):
        host_tags = {}
        for t in self.node:
            host_tags.setdefault(t.attrib.get('name'), t.text)
        return host_tags


class ReportHost:
    def __init__(self, node):
        self.node = node

    @property
    def name(self) -> str:
        return self.node.get("name")

    @property
    def host_properties(self) -> HostProperties:
        return HostProperties(self.node.find("HostProperties"))

    @property
    def report_items(self) -> List[ReportItem]:
        return [ReportItem(i) for i in self.node.findall("ReportItem")]


class Report:

    def __init__(self, node):
        self.node = node

    @property
    def name_attr(self) -> str:
        return self.node.get("name")

    @property
    def report_hosts(self) -> List[ReportHost]:
        return [ReportHost(i) for i in self.node.findall('ReportHost')]



import xml.etree.ElementTree as ET

from faraday_plugins.plugins.plugin import PluginXMLFormat

__author__ = "@rfocke and @pasta <3"
__copyright__ = "Copyright (c) 2021, Faradaysec LLC"
__credits__ = ["Roberto Focke", "Javier Aguinaga"]
__license__ = "GPL"
__version__ = "0.8"
__mantainer__ = "@rfocke"
__status__ = "Development"


class VulnSoftNipper:
    def __init__(self, **kwargs):
        self.name = ''
        self.data = ''
        self.device = ''
        self.cvss2 = {}
        self.refs = []


class VulnerabilityNipper:
    def __init__(self, **kwargs):
        self.name = ''
        self.rating = ''
        self.recommendation = ''
        self.affected_devices = []
        self.section = ''
        self.name2 = ''
        self.data = ''
        self.recommendation2 = ''


class NipperParser:
    def __init__(self, output, debug=False):
        self.vulns_first = []
        self.vulns_audit = []
        self.vulns_thirds = []
        self.debug = debug

        self.tree = ET.fromstring(output)
        self.report_tree = self.tree.find(
            "report/part/[@index='2']/section/[@title='Recommendations']/table/[@title='Security Audit recommendations list']/tablebody")
        self.process_xml()

    def process_xml(self):
        if not self.report_tree:
            return

        for tablerow in self.report_tree:
            for i, tablecell in enumerate(tablerow.findall('tablecell')):
                if len(tablecell.findall('item')) == 1:
                    if i == 0:  # Item
                        vuln = VulnerabilityNipper()
                        vuln.name = tablecell.find('item').text
                    elif i == 1:  # Rating
                        vuln.rating = tablecell.find('item').text
                    elif i == 2:  # Recommendations
                        vuln.recommendation = tablecell.find('item').text
                    elif i == 3:  # Affected devices (with 1 element only)
                        vuln.affected_devices = []
                        vuln.affected_devices.append(tablecell.find('item').text)
                    elif i == 4:  # Section
                        subdetail = tablecell.find('item').text
                        vuln.section = subdetail

                        path = "./report/part/[@index='2']/section/[@index='" + subdetail + "']"
                        for detail in self.tree.findall(path):
                            # nombre de la vuln
                            vuln.name2 = detail.attrib.get('title')

                        if vuln.name2 != vuln.name:
                            pass

                        path = "./report/part/[@index='2']/section/[@index='" + subdetail + "']/section/[@index='" + subdetail + ".2']"
                        for detail in self.tree.findall(path):
                            # data de la vuln
                            vuln.data = detail.find('text').text

                        path = "./report/part/[@index='2']/section/[@index='" + subdetail + "']/section/[@index='" + subdetail + ".5']"
                        for detail in self.tree.findall(path):
                            # recomendacion de la vuln
                            vuln.recommendation2 = detail.find('text').text

                        self.vulns_first.append(vuln)  # <- GUARDADO
                elif len(tablecell.findall('item')) > 1 and i == 3:
                    # affected devices
                    vuln.affected_devices = []
                    for item in tablecell.findall('item'):
                        vuln.affected_devices.append(item.text)

        # parseo vuln de software
        report_tree = self.tree.find("./report/part/[@title='Vulnerability Audit']")
        for itemv in report_tree:
            vuln_soft = VulnSoftNipper()
            # nombre de la vuln

            vuln_soft.name = itemv.attrib.get('title')
            cvss2_vector = itemv.find('infobox/infodata/[@label="CVSSv2 Base"]')
            vuln_soft.cvss2["vector_string"] = cvss2_vector.text.split(' ')[0] if cvss2_vector is not None else None
            for itemvv in itemv:
                if itemvv.attrib.get('title') == 'Summary':
                    for i in itemvv:
                        # data de la vuln
                        vuln_soft.data = i.text
                if itemvv.attrib.get('title') == 'Affected Device':
                    for i in itemvv:
                        # data del device
                        aux = i.text.split('The')[1]
                        vuln_soft.device = aux.split(' may be affected by this security vulnerability')[0]
                if itemvv.attrib.get('title') == 'References':
                    # referencias de la vuln
                    vuln_soft.refs = []
                    for texto in itemvv.findall('list/listitem/weblink'):
                        vuln_soft.refs.append(texto.text)

            self.vulns_audit.append(vuln_soft)


class NipperPlugin(PluginXMLFormat):
    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.extension = ".xml"
        self.identifier_tag = "document"
        self.identifier_tag_attributes = {'nipperstudio'}
        self.id = "Nipper"
        self.name = "Nipper XML Output Plugin"
        self.plugin_version = "0.9"
        self.version = "0.9"
        self.framewor_version = "1.0.1"
        self.options = None

    def parseOutputString(self, output):
        parser = NipperParser(output, debug=False)
        for vuln in parser.vulns_first:
            for device in vuln.affected_devices:
                ip = self.resolve_hostname(device)
                h_id = self.createAndAddHost(ip, hostnames=device)
                self.createAndAddVulnToHost(h_id,
                                            name=vuln.name,
                                            desc=vuln.data,
                                            severity=vuln.rating,
                                            resolution=vuln.recommendation,
                                            data=vuln.data,
                                            ref=[],
                                            policyviolations=[],
                                            cve=[vuln.name]
                                            )
        for vuln in parser.vulns_audit:
            if vuln.data:
                ip = self.resolve_hostname(device)
                h_id = self.createAndAddHost(ip, hostnames=vuln.device)
                self.createAndAddVulnToHost(h_id,
                                            name=vuln.name,
                                            desc=vuln.data,
                                            severity='',
                                            resolution='',
                                            data=vuln.data,
                                            ref=vuln.refs,
                                            cve=[vuln.name],
                                            cvss2=vuln.cvss2
                                            )


def createPlugin(*args, **kwargs):
    return NipperPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2019  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import json
from faraday_plugins.plugins.plugin import PluginJsonFormat
from urllib.parse import urlparse
import os


__author__ = "Blas Moyano"
__copyright__ = "Copyright (c) 2019, Infobyte LLC"
__credits__ = ["Blas Moyano"]
__license__ = ""
__version__ = "0.0.1"
__maintainer__ = "Blas Moyano"
__email__ = "bmoyano@infobytesec.com"
__status__ = "Development"


class SourceclearJsonParser:
    def __init__(self, json_output):
        self.json_data = json.loads(json_output)

    def parse_url(self, url):
        url_parse = urlparse(url)
        protocol = url_parse.scheme
        hostname = url_parse.netloc
        port = url_parse.port

        if port is None:
            if protocol == 'https':
                port = 443
            elif protocol == 'http':
                if not port:
                    port = 80

        return {'protocol': protocol, 'hostname': hostname, 'port': port}


class SourceclearPlugin(PluginJsonFormat):
    """ Handle the Sourceclear tool. Detects the output of the tool
    and adds the information to Faraday.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "sourceclear"
        self.name = "Sourceclear"
        self.plugin_version = "0.1"
        self.version = "0.0.1"
        self.json_keys = {"metadata", "records"}

    def parseOutputString(self, output, debug=False):
        parser = SourceclearJsonParser(output)

        for records in parser.json_data['records']:
            vulns = records['vulnerabilities']
            libraries = records['libraries']

            for vuln in vulns:
                v_name = vuln['title']
                v_desc = vuln['overview']
                v_data = vuln['libraries']
                v_website = vuln['_links']['html']
                url_data = parser.parse_url(v_website)
                for refs in vuln['libraries']:
                    ref = refs['_links']['ref']
                    num_versions = ref.find("/versions")
                    _, num_libraries = os.path.split(ref[:num_versions])
                    name_librarie = libraries[int(num_libraries)]['name']
                    version_librarie = libraries[int(num_libraries)]['versions'][0]['version']
                    host_name = f'{name_librarie}{version_librarie}'

                h_id = self.createAndAddHost(name=host_name, scan_template=records['metadata']['recordType'])
                s_id = self.createAndAddServiceToHost(h_id, "Sourceclear", protocol=url_data['protocol'],
                                                      ports=url_data['port'], status='open')
                self.createAndAddVulnWebToService(h_id, s_id, name=v_name, desc=v_desc, data=v_data,
                                                  website=v_website)


def createPlugin(*args, **kwargs):
    return SourceclearPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2018 Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import re

from faraday_plugins.plugins.plugin import PluginXMLFormat
from faraday_plugins.plugins.plugins_utils import get_vulnweb_url_fields

import xml.etree.ElementTree as ET


def cleanhtml(raw_html):
        cleanr = re.compile('<.*?>')
        cleantext = re.sub(cleanr, '', raw_html)
        return cleantext


class WebInspectParser():

    def __init__(self, output):
        self.xml = ET.fromstring(output)
        self.issues = self.xml.findall("Issues/Issue")

    def parse_severity(self, severity):

        severity_dict = {
            "0": "info",
            "1": "low",
            "2": "med",
            "3": "high",
            "4": "critical"}

        result = severity_dict.get(severity)
        if not result:
            return "info"
        else:
            return result

    def return_text(self, tag,element):
        try:
            text = element.find(tag).text
            return text
        except:
            return ""

    def parse(self):

        map_objects_fields = {
            "Name": ["Vuln", "name"],
            "URL": ["Vuln", "website"],
            "Scheme": ["Service", "name"],
            "Host": ["Host", "name"],
            "Port": ["Service", "port"],
            "AttackMethod": ["Vuln", "method"],
            "VulnerableSession": ["Vuln", "request"],
            "VulnerabilityID": ["Vuln", "reference"],
            "RawResponse": ["Vuln", "response"],
            "Summary": ["Vuln", "description"],
            "Implication": ["Vuln", "data"],
            "Fix": ["Vuln", "resolution"],
            "Reference Info": ["Vuln", "reference"],
            "Severity": ["Vuln", "severity"]
        }

        result = []
        for issue in self.issues:

            obj = {
                "Host" : {},
                "Service" : {},
                "Interface" : {},
                "Vuln": {
                    "reference" : []}
            }

            for tag, obj_property in map_objects_fields.items():

                value = self.return_text(tag,issue)

                if value is not None:

                    faraday_obj_name = obj_property[0]
                    faraday_field = obj_property[1]
                    if faraday_field == "reference":
                        obj[faraday_obj_name].get("reference").append(value)
                    else:
                        obj[faraday_obj_name].update({faraday_field:value})

            # This for loads Summary, Implication, Fix and Reference
            for section in issue.findall("ReportSection"):

                try:
                    field = section.find("Name").text
                    value = section.find("SectionText").text

                    faraday_obj_name = map_objects_fields.get(field)[0]
                    faraday_field = map_objects_fields.get(field)[1]
                except:
                    continue

                if faraday_field == "reference" and value != "":
                    obj[faraday_obj_name].get("reference").append(cleanhtml(value))
                else:
                    obj[faraday_obj_name].update({faraday_field:value})

            result.append(obj)
        return result


class WebInspectPlugin(PluginXMLFormat):
    """
    This plugin handles WebInspect reports.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Webinspect"
        self.name = "Webinspect"
        self.plugin_version = "0.0.1"
        self.version = "1.0.0"
        self.identifier_tag = ["Scan"]

    def parseOutputString(self, output):

        parser = WebInspectParser(output)
        vulns = parser.parse()

        for vuln in vulns:

            host_id = self.createAndAddHost(vuln.get("Host").get("name"))
            service_id = self.createAndAddServiceToHost(host_id, vuln.get("Service").get("name"),
                                                        protocol=vuln.get("Service").get("name"),
                                                        ports=[vuln.get("Service").get("port")])

            self.createAndAddVulnWebToService(
                host_id, service_id,
                vuln.get("Vuln").get("name"),
                website=get_vulnweb_url_fields(vuln.get("Vuln").get("website")).get("website"),
                path=get_vulnweb_url_fields(vuln.get("Vuln").get("website")).get("path"),
                query=get_vulnweb_url_fields(vuln.get("Vuln").get("website")).get("query"),
                method=vuln.get("Vuln").get("method"),
                request=vuln.get("Vuln").get("request"),
                ref=list(filter(None ,vuln.get("Vuln").get("reference"))),
                response=vuln.get("Vuln").get("response"),
                desc=cleanhtml(vuln.get("Vuln").get("description")),
                resolution=cleanhtml(vuln.get("Vuln").get("resolution")),
                severity=parser.parse_severity(vuln.get("Vuln").get("severity"))
            )


def createPlugin(*args, **kwargs):
    return WebInspectPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import argparse
import re
import shlex
import socket

from faraday_plugins.plugins.plugin import PluginBase


class AmapPlugin(PluginBase):
    """ Example plugin to parse amap output."""

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Amap"
        self.name = "Amap Output Plugin"
        self.plugin_version = "0.0.3"
        self.version = "5.4"
        self.options = None
        self._current_output = None
        self._command_regex = re.compile(r'^(amap|sudo amap)\s+.*?')
        self._use_temp_file = True
        self._hosts = []
        self.args = None

    def parseOutputString(self, output):
        services = {}
        for line in output.split('\n'):
            if line.startswith('#'):
                continue
            fields = self.get_info(line)
            if len(fields) < 6:
                continue
            address = fields[0]
            port = fields[1]
            protocol = fields[2]
            port_status = fields[3]
            identification = fields[5]
            printable_banner = fields[6]
            if port in services.keys():
                if identification != 'unidentified':
                    services[port][5] += ', ' + identification
            else:
                services[port] = [
                    address,
                    port,
                    protocol,
                    port_status,
                    None,
                    identification,
                    printable_banner,
                    None]

            if address != self.args.m:
                hostnames = [self.args.m]
            else:
                hostnames = None
            h_id = self.createAndAddHost(address, hostnames=hostnames)

        for key in services:
            service = services.get(key)
            self.createAndAddServiceToHost(
                h_id,
                service[5],
                service[2],
                ports=[service[1]],
                status=service[3],
                description=service[6])

        return True

    file_arg_re = re.compile(r"^.*(-o \s*[^\s]+\s+(?:-m|)).*$")

    def get_info(self, data):
        if self.args.__getattribute__("6"):
            f = re.search(
                r"^\[(.*)\]:(.*):(.*):(.*):(.*):(.*):(.*):(.*)",
                data)

            return [
                f.group(1),
                f.group(2),
                f.group(3),
                f.group(4),
                f.group(5),
                f.group(6),
                f.group(7),
                f.group(8)] if f else []

        else:
            return data.split(':')

    def get_ip_6(self, host, port=0):
        alladdr = socket.getaddrinfo(host, port)
        ip6 = list(filter(
            lambda x: x[0] == socket.AF_INET6,
            alladdr))

        return ip6[0][4][0]

    def processCommandString(self, username, current_path, command_string):
        """
        Adds the -m parameter to get machine readable output.
        """
        super().processCommandString(username, current_path, command_string)
        arg_match = self.file_arg_re.match(command_string)
        parser = argparse.ArgumentParser()
        parser.add_argument('-6', action='store_true')
        parser.add_argument('-o')
        parser.add_argument('-m')
        if arg_match is None:
            final = re.sub(
                r"(^.*?amap)",
                r"\1 -o %s -m " % self._output_file_path,
                command_string)
        else:
            final = re.sub(
                arg_match.group(1),
                r"-o %s -m " % self._output_file_path,
                command_string)

        cmd = shlex.split(re.sub(r'\-h|\-\-help', r'', final))
        if "-6" in cmd:
            cmd.remove("-6")
            cmd.insert(1, "-6")

        if len(cmd) > 4:
            try:
                self.args, unknown = parser.parse_known_args(cmd)
            except SystemExit:
                pass

        return final


def createPlugin(*args, **kwargs):
    return AmapPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re
import xml.etree.ElementTree as ET
from urllib.parse import urlparse

from faraday_plugins.plugins.plugin import PluginXMLFormat

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


def get_urls(string):
    if isinstance(string, bytes):
        string_decode = string.decode("utf-8")
        urls = re.findall(r'(?:(?:https?|ftp):\/\/)?[\w/\-?=%.]+\.[\w/\-?=%.]+', string_decode)
    else:
        urls = re.findall(r'(?:(?:https?|ftp):\/\/)?[\w/\-?=%.]+\.[\w/\-?=%.]+', string)
    return urls


class NetsparkerCloudXmlParser:
    """
    The objective of this class is to parse an xml file generated by the netsparkercloud tool.

    TODO: Handle errors.
    TODO: Test netsparkercloud output version. Handle what happens if the parser doesn't support it.
    TODO: Test cases.

    @param netsparkercloud_xml_filepath A proper xml generated by netsparkercloud
    """

    def __init__(self, xml_output):
        self.filepath = xml_output
        tree = self.parse_xml(xml_output)
        if tree:
            self.items = self.get_items(tree)
        else:
            self.items = []

    def parse_xml(self, xml_output):
        """
        Open and parse an xml file.

        TODO: Write custom parser to just read the nodes that we need instead of
        reading the whole file.

        @return xml_tree An xml tree instance. None if error.
        """
        try:
            tree = ET.fromstring(xml_output)
        except SyntaxError as err:
            self.logger.error(f"SyntaxError: {err}. {xml_output}")
            return None
        return tree

    def get_items(self, tree):
        """
        @return items A list of Host instances
        """
        for node in tree.findall("vulnerabilities/vulnerability"):
            yield Item(node)


class Item:
    """
    An abstract representation of a Item


    @param item_node A item_node taken from an netsparkercloud xml tree
    """

    def re_map_severity(self, severity):
        if severity == "Important":
            return "high"
        return severity

    def __init__(self, item_node, encoding="ascii"):
        self.node = item_node
        self.url = urlparse(self.get_text_from_subnode("url"))
        self.protocol = self.url.scheme
        self.hostname = self.url.netloc
        self.port = self.url.port
        if self.port is None:
            self.port = '80'
        self.type = self.get_text_from_subnode("type")
        self.name = self.get_text_from_subnode("name")
        self.severity = self.re_map_severity(self.get_text_from_subnode("severity"))
        self.certainty = self.get_text_from_subnode("certainty")
        self.node = item_node.find("http-request")
        self.method = self.get_text_from_subnode("method")
        self.request = self.get_text_from_subnode("content")
        self.param = ""
        self.paramval = ""
        for p in self.node.findall("parameters/parameter"):
            self.param = p.get('name')
            self.paramval = p.get('value')

        self.node = item_node.find("http-response")
        self.response = self.get_text_from_subnode("content")
        self.extra = []
        for v in item_node.findall("extra-information/info"):
            self.extra.append(v.get('name') + ":" + v.get('value'))

        self.node = item_node.find("classification")
        self.owasp = self.get_text_from_subnode("owasp")
        self.wasc = self.get_text_from_subnode("wasc")
        self.cwe = self.get_text_from_subnode("cwe")
        self.capec = self.get_text_from_subnode("capec")
        self.pci = self.get_text_from_subnode("pci31")
        self.pci2 = self.get_text_from_subnode("pci32")
        self.hipaa = self.get_text_from_subnode("hipaa")

        self.ref = []
        if self.cwe:
            self.cwe = [f"CWE-{self.cwe}"]
        if self.owasp:
            self.ref.append(f"OWASP-{self.owasp}")

        self.node = item_node
        self.remedyreferences = self.get_text_from_subnode("remedy-references")
        self.externalreferences = self.get_text_from_subnode("external-references")
        if self.remedyreferences:
            for u in get_urls(self.remedyreferences):
                self.ref.append(u)
        if self.externalreferences:
            for u in get_urls(self.externalreferences):
                self.ref.append(u)

        self.impact = self.get_text_from_subnode("impact")
        self.remedialprocedure = self.get_text_from_subnode("remedial-procedure")
        self.remedialactions = self.get_text_from_subnode("remedial-actions")
        self.exploitationskills = self.get_text_from_subnode("exploitation-skills")
        self.proofofconcept = self.get_text_from_subnode("proof-of-concept")

        self.resolution = "Remerdial Procedure: {} \nRemedial Actions: {}".format(self.remedialprocedure,
                                                                                  self.remedialactions)

        self.desc = self.get_text_from_subnode("description")
        self.desc = "\nImpact: {} \nExploitation Skills: {} \nProof of concept: {} \nWASC: {}  \nPCI31: {} \nPCI32: {}" \
                    " \nCAPEC: {} \nHIPA: {} \nExtra: {}".format(self.impact, self.exploitationskills,
                                                                 self.proofofconcept, self.wasc, self.pci, self.pci2,
                                                                 self.capec, self.hipaa, self.extra)

    def get_text_from_subnode(self, subnode_xpath_expr):
        """
        Finds a subnode in the host node and the retrieves a value from it.

        @return An attribute value
        """
        if self.node:
            sub_node = self.node.find(subnode_xpath_expr)
            if sub_node is not None:
                return sub_node.text
        return None


class NetsparkerCloudPlugin(PluginXMLFormat):
    """
    Example plugin to parse netsparkercloud output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = "netsparker-cloud"
        self.id = "NetsparkerCloud"
        self.name = "NetsparkerCloud XML Output Plugin"
        self.plugin_version = "0.0.1"
        self.version = "NetsparkerCloud"
        self.framework_version = "1.0.0"
        self.options = None

    def parseOutputString(self, output):
        parser = NetsparkerCloudXmlParser(output)
        first = True
        for i in parser.items:
            if first:
                ip = self.resolve_hostname(i.hostname)
                h_id = self.createAndAddHost(ip, hostnames=[i.hostname])
                s_id = self.createAndAddServiceToHost(h_id, i.protocol, ports=[i.port], status="open")
                first = False
            v_id = self.createAndAddVulnWebToService(h_id, s_id, i.name, ref=i.ref, website=i.hostname,
                                                     severity=i.severity, desc=i.desc, path=i.url.path, method=i.method,
                                                     request=i.request, response=i.response, resolution=i.resolution,
                                                     pname=i.param, cwe=i.cwe)
        del parser


def createPlugin(*args, **kwargs):
    return NetsparkerCloudPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re
import os

from faraday_plugins.plugins.plugin import PluginBase


__author__ = "Javier Victor Mariano Bruno"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Javier Victor Mariano Bruno"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Javier Victor Mariano Bruno"
__email__ = "mbruno@infobytesec.com"
__status__ = "Development"


class CmdFtpPlugin(PluginBase):
    """
    This plugin handles ftp command.
    Basically detects if user was able to connect to a device
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "ftp"
        self.name = "Ftp"
        self.plugin_version = "0.0.1"
        self.version = "0.17"
        self.framework_version = "1.0.0"
        self.options = None
        self._current_output = None
        self._command_regex = re.compile(r'^ftp\s+.*?')
        self._host_ip = None
        self._port = "21"
        self._info = 0
        self._version = None



    def parseOutputString(self, output):

        host_info = re.search(r"Connected to (.+)\.", output)
        banner = re.search(r"220?([\w\W]+)$", output)
        if re.search("Connection timed out", output) is None and host_info is not None:
            hostname = host_info.group(1)
            ip_address = self.resolve_hostname(hostname)
            self._version = banner.groups(0) if banner else ""
            h_id = self.createAndAddHost(ip_address, hostnames=[hostname])
            s_id = self.createAndAddServiceToHost(
                h_id,
                "ftp",
                "tcp",
                ports=[self._port],
                status="open")
        return True

    def processCommandString(self, username, current_path, command_string):
        """
        """
        super().processCommandString(username, current_path, command_string)
        count_args = command_string.split()
        c = count_args.__len__()
        self._port = "21"
        if re.search(r"[\d]+", count_args[c - 1]):
            self._port = count_args[c - 1]


def createPlugin(*args, **kwargs):
    return CmdFtpPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2015  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import xml.etree.ElementTree as ET
import zipfile

from faraday_plugins.plugins.plugin import PluginZipFormat

__author__ = "Ezequiel Tavella"
__copyright__ = "Copyright (c) 2015, Infobyte LLC"
__credits__ = ["Ezequiel Tavella"]
__license__ = ""
__version__ = "1.0.1"
__maintainer__ = "Ezequiel Tavella"
__status__ = "Development"


def readMtgx(mtgl_file):
    maltego_file_graph = "Graphs/Graph1.graphml"
    xml_graph = ET.parse(mtgl_file.open(maltego_file_graph))
    mtgl_file.close()
    return xml_graph


def readMtgl(mtgl_file):
    try:
        maltego_file_company = "Entities/maltego.Company.entity"
        maltego_file_dns = "Entities/maltego.DNSName.entity"
        maltego_file_domain = "Entities/maltego.Domain.entity"
        maltego_file_email = "Entities/maltego.EmailAddress.entity"
        maltego_file_ipv4 = "Entities/maltego.IPv4Address.entity"
        maltego_file_location = "Entities/maltego.Location.entity"
        maltego_file_mxrecord = "Entities/maltego.MXRecord.entity"
        maltego_file_nsrecord = "Entities/maltego.NSRecord.entity"
        maltego_file_organization = "Entities/maltego.Organization.entity"
        maltego_file_person = "Entities/maltego.Person.entity"
        maltego_file_number = "Entities/maltego.PhoneNumber.entity"
        maltego_file_website = "Entities/maltego.Website.entity"
        check_files = {}

        if maltego_file_company in mtgl_file.namelist():
            xml_company = ET.parse(mtgl_file.open(maltego_file_company))
            check_files.update({"company": xml_company})

        if maltego_file_dns in mtgl_file.namelist():
            xml_dns = ET.parse(mtgl_file.open(maltego_file_dns))
            check_files.update({"domain": xml_dns})

        if maltego_file_domain in mtgl_file.namelist():
            xml_domain = ET.parse(mtgl_file.open(maltego_file_domain))
            check_files.update({"domain": xml_domain})

        if maltego_file_email in mtgl_file.namelist():
            xml_email = ET.parse(mtgl_file.open(maltego_file_email))
            check_files.update({"email": xml_email})

        if maltego_file_ipv4 in mtgl_file.namelist():
            xml_ipv4 = ET.parse(mtgl_file.open(maltego_file_ipv4))
            check_files.update({"ipv4": xml_ipv4})

        if maltego_file_location in mtgl_file.namelist():
            xml_location = ET.parse(mtgl_file.open(maltego_file_location))
            check_files.update({"location": xml_location})

        if maltego_file_mxrecord in mtgl_file.namelist():
            xml_mxrecord = ET.parse(mtgl_file.open(maltego_file_mxrecord))
            check_files.update({"mxrecord": xml_mxrecord})

        if maltego_file_nsrecord in mtgl_file.namelist():
            xml_nsrecord = ET.parse(mtgl_file.open(maltego_file_nsrecord))
            check_files.update({"nsrecord": xml_nsrecord})

        if maltego_file_organization in mtgl_file.namelist():
            xml_organization = ET.parse(mtgl_file.open(maltego_file_organization))
            check_files.update({"organization": xml_organization})

        if maltego_file_person in mtgl_file.namelist():
            xml_person = ET.parse(mtgl_file.open(maltego_file_person))
            check_files.update({"person": xml_person})

        if maltego_file_number in mtgl_file.namelist():
            xml_number = ET.parse(mtgl_file.open(maltego_file_number))
            check_files.update({"number": xml_number})

        if maltego_file_website in mtgl_file.namelist():
            xml_web = ET.parse(mtgl_file.open(maltego_file_website))
            check_files.update({"web": xml_web})

    except zipfile.BadZipFile:
        return None

    mtgl_file.close()
    return check_files


class Host:

    def __init__(self):
        self.ip = ""
        self.node_id = ""
        self.dns_name = set()
        self.website = ""
        self.netblock = ""
        self.location = ""
        self.mx_record = ""
        self.ns_record = ""


class MaltegoParser:

    def __init__(self, xml_file, extension, resolve_hostname):

        self.resolve_hostname = resolve_hostname
        if extension == '.mtgx':
            self.xml = readMtgx(xml_file)
            self.nodes = self.xml.findall(
                "{http://graphml.graphdrawing.org/xmlns}graph/"
                "{http://graphml.graphdrawing.org/xmlns}node")
            self.edges = self.xml.findall(
                "{http://graphml.graphdrawing.org/xmlns}graph/"
                "{http://graphml.graphdrawing.org/xmlns}edge")

            self.list_hosts = []
            self.relations = {}
        elif extension == '.mtgl':
            self.xml = readMtgl(xml_file)

    def getRelations(self):
        """
        Get relations between nodes.
        Two ways: Source-> Target
        Source <- Target
        """
        for edge in self.edges:

            source = edge.get("source")
            target = edge.get("target")

            if source not in self.relations:
                self.relations.update({source: [target]})

            if target not in self.relations:
                self.relations.update({target: [source]})

            values = self.relations[source]
            values.append(target)
            self.relations.update({source: values})

            values = self.relations[target]
            values.append(source)
            self.relations.update({target: values})

    def getIpAndId(self, node):
        # Find node ID and maltego entity
        node_id = node.get("id")
        entity = node.find(
            "{http://graphml.graphdrawing.org/xmlns}data/"
            "{http://maltego.paterva.com/xml/mtgx}MaltegoEntity")

        # Check if is IPv4Address
        if entity.get("type") not in ("maltego.IPv4Address", "maltego.Domain", "maltego.Website"):
            return None

        # Get IP value
        value = entity.find(
            "{http://maltego.paterva.com/xml/mtgx}Properties/"
            "{http://maltego.paterva.com/xml/mtgx}Property/"
            "{http://maltego.paterva.com/xml/mtgx}Value")
        if entity.get("type") in ("maltego.Domain", "maltego.Website"):
            ip = self.resolve_hostname(value.text)
            hostname = value.text
        else:
            ip = value.text
            hostname = None
        return {"node_id": node_id, "ip": ip, "hostname": hostname}

    def getNode(self, node_id):

        # Get node, filter by id
        for node in self.nodes:

            if node.get("id") == node_id:
                return node

    def getType(self, node):

        # Get type of this node
        entity = node.find(
            "{http://graphml.graphdrawing.org/xmlns}data/"
            "{http://maltego.paterva.com/xml/mtgx}MaltegoEntity")

        return entity.get("type")

    def getWebsite(self, target_node):

        # Parse Website Entity
        result = {"name": "", "ssl_enabled": "", "urls": ""}

        props = target_node.find(
            "{http://graphml.graphdrawing.org/xmlns}data/"
            "{http://maltego.paterva.com/xml/mtgx}MaltegoEntity/"
            "{http://maltego.paterva.com/xml/mtgx}Properties")

        for prop in props:

            name_property = prop.get("name")
            value = prop.find(
                "{http://maltego.paterva.com/xml/mtgx}Value").text

            if name_property == "fqdn":
                result["name"] = value
            elif name_property == "website.ssl-enabled":
                result["ssl_enabled"] = value
            elif name_property == "URLS":
                result["urls"] = value

        return result

    def getNetBlock(self, target_node):

        # Parse Netblock Entity
        result = {"ipv4_range": "", "network_owner": "", "country": ""}

        props = target_node.find(
            "{http://graphml.graphdrawing.org/xmlns}data/"
            "{http://maltego.paterva.com/xml/mtgx}MaltegoEntity/"
            "{http://maltego.paterva.com/xml/mtgx}Properties")

        for prop in props:

            name_property = prop.get("name")
            value = prop.find(
                "{http://maltego.paterva.com/xml/mtgx}Value").text

            if name_property == "ipv4-range":
                result["ipv4_range"] = value
            elif name_property == "description":
                result["network_owner"] = value
            elif name_property == "country":
                result["country"] = value

        return result

    def getLocation(self, target_node):

        # Parse Location Entity
        result = {"name": "", "area": "", "country_code": "", "longitude": "", "latitude": "", "area_2": ""}

        # Get relations with other nodes
        node_relations = self.relations[target_node.get("id")]

        # Find location node based in relation with netblock node.
        located = False
        for node_id in node_relations:

            target_node = self.getNode(node_id)
            if self.getType(target_node) == "maltego.Location":
                located = True
                break

        if not located:
            return None

        # Get properties and update data
        props = target_node.find(
            "{http://graphml.graphdrawing.org/xmlns}data/"
            "{http://maltego.paterva.com/xml/mtgx}MaltegoEntity/"
            "{http://maltego.paterva.com/xml/mtgx}Properties")

        for prop in props:

            name_property = prop.get("name")
            value = prop.find(
                "{http://maltego.paterva.com/xml/mtgx}Value").text

            if name_property == "location.name":
                result["name"] = value
            elif name_property == "location.area":
                result["area"] = value
            elif name_property == "countrycode":
                result["country_code"] = value
            elif name_property == "longitude":
                result["longitude"] = value
            elif name_property == "latitude":
                result["latitude"] = value
            elif name_property == "area":
                result["area_2"] = value

        return result

    def getValue(self, target_node):

        # Parse Entity
        result = {"value": ""}

        value = target_node.find(
            "{http://graphml.graphdrawing.org/xmlns}data/"
            "{http://maltego.paterva.com/xml/mtgx}MaltegoEntity/"
            "{http://maltego.paterva.com/xml/mtgx}Properties/"
            "{http://maltego.paterva.com/xml/mtgx}Property/"
            "{http://maltego.paterva.com/xml/mtgx}Value")

        result["value"] = value.text
        return result

    def parse(self):

        self.getRelations()

        for node in self.nodes:
            # Get IP Address if not continue with other node...
            result = self.getIpAndId(node)
            if not result:
                continue

            # Create host with values by default
            host = Host()
            host.ip = result.get("ip")
            host.node_id = result.get("node_id")
            if result.get("hostname"):
                host.dns_name.add(result.get("hostname"))
            # Get relations with other nodes
            node_relations = self.relations[host.node_id]

            for node_id in node_relations:

                # Get target node and type of node.
                target_node = self.getNode(node_id)
                target_type = self.getType(target_node)

                # Check type of node y add data to host...
                if target_type in ("maltego.DNSName", "maltego.Domain"):
                    host.dns_name.add(self.getValue(target_node)['value'])
                elif target_type == "maltego.Website":
                    host.website = self.getWebsite(target_node)
                elif target_type == "maltego.Netblock":
                    host.netblock = self.getNetBlock(target_node)
                    # Get location based in relation: netblock -> location
                    host.location = self.getLocation(target_node)
                elif target_type == "maltego.MXRecord":
                    host.mx_record = self.getValue(target_node)
                elif target_type == "maltego.NSRecord":
                    host.ns_record = self.getValue(target_node)

            self.list_hosts.append(host)

        return self.list_hosts

    def getInfoMtgl(self, xml, name):
        sample_value = xml.findall(f'Properties/Fields/Field[@name="{name}"]')
        for data in sample_value:
            mtgl_data = data.find('SampleValue').text
        return mtgl_data


class MaltegoPlugin(PluginZipFormat):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = "maltego"
        self.id = "Maltego"
        self.name = "Maltego MTGX & MTGL Output Plugin"
        self.plugin_version = "1.0.1"
        self.version = "Maltego 3.6"
        self.framework_version = "1.0.0"
        self.extension = [".mtgl", ".mtgx"]
        self.files_list = {"Graphs/Graph1.graphml", "Entities/maltego.Company.entity",
                           "Entities/maltego.DNSName.entity", "Entities/maltego.Domain.entity",
                           "Entities/maltego.EmailAddress.entity", "Entities/maltego.IPv4Address.entity",
                           "Entities/maltego.Location.entity", "Entities/maltego.MXRecord.entity",
                           "Entities/maltego.Organization.entity", "Entities/maltego.NSRecord.entity",
                           "Entities/maltego.Person.entity", "Entities/maltego.PhoneNumber.entity",
                           "Entities/maltego.Website.entity", "Entities/maltego.Hash.entity",
                           "Entities/maltego.hashtag.entity", "Entities/maltego.TwitterUserList.entity"}

    def parseOutputString(self, output):
        if 'Graphs/Graph1.graphml' in output.namelist():
            maltego_parser = MaltegoParser(output, self.extension[1], resolve_hostname=self.resolve_hostname)
            hosts = maltego_parser.parse()
            if not hosts:
                self.logger.warning("No hosts data found in maltego report")
                pass
            else:
                for host in hosts:
                    if host.ip is None:
                        ip = '0.0.0.0'
                        self.logger.warning("Unknown IP")
                    else:
                        ip = host.ip
                    host_id = self.createAndAddHost(ip, hostnames=list(host.dns_name))
                    # Create note with NetBlock information
                    if host.netblock:
                        try:
                            text = f'Network owner:\n {host.netblock["network_owner"]} ' \
                                   f'Country:\n {host.netblock["country"]}'
                        except TypeError:
                            text = "unknown"

                        self.createAndAddNoteToHost(host_id=host_id, name="Netblock Information",
                                                    text=text.encode('ascii', 'ignore'))

                    # Create note with host location
                    if host.location:
                        try:
                            text = f'Location:\n {host.location["name"]} \nArea:\n {host.location["area"]} ' \
                                   f'\nArea 2:\n {host.location["area_2"]} ' \
                                   f'\nCountry_code:\n {host.location["country_code"]} ' \
                                   f'\nLatitude:\n {host.location["latitude"]} \nLongitude:\n {host.location["longitude"]}'
                        except TypeError:
                            text = "unknown"

                        self.createAndAddNoteToHost(host_id=host_id, name="Location Information",
                                                    text=text.encode('ascii', 'ignore'))

                    # Create service web server
                    if host.website:
                        try:
                            description = f'SSL Enabled: {host.website["ssl_enabled"]}'
                        except TypeError:
                            description = "unknown"

                        service_id = self.createAndAddServiceToHost(host_id=host_id, name=host.website["name"],
                                                                    protocol="TCP:HTTP", ports=[80],
                                                                    description=description)

                        try:
                            text = f'Urls: \n {host.website["urls"]}'
                            self.createAndAddNoteToService(host_id=host_id, service_id=service_id, name="URLs",
                                                           text=text.encode('ascii', 'ignore'))
                        except TypeError:
                            pass

                    if host.mx_record:
                        self.createAndAddServiceToHost(host_id=host_id, name=host.mx_record["value"], protocol="SMTP",
                                                       ports=[25], description="E-mail Server")

                    if host.ns_record:
                        self.createAndAddServiceToHost(host_id=host_id, name=host.ns_record["value"], protocol="DNS",
                                                       ports=[53], description="DNS Server")
        else:
            maltego_parser = MaltegoParser(output, self.extension[0], resolve_hostname=self.resolve_hostname)
            if not maltego_parser.xml.get('domain') or not maltego_parser.xml.get('ipv4'):
                return
            if maltego_parser.xml.get('domain'):
                hostnames = maltego_parser.getInfoMtgl(maltego_parser.xml['domain'], 'fqdn')
            else:
                hostnames = None
            if maltego_parser.xml.get('ipv4'):
                host_ip = maltego_parser.getInfoMtgl(maltego_parser.xml['ipv4'], 'ipv4-address')
                host_id = self.createAndAddHost(name=host_ip, hostnames=hostnames)
            else:
                host_ip = '0.0.0.0'
                host_id = self.createAndAddHost(name=host_ip, hostnames=hostnames)

            if maltego_parser.xml.get('location'):
                location_name = maltego_parser.getInfoMtgl(maltego_parser.xml['location'], 'location.name')
                location_area = maltego_parser.getInfoMtgl(maltego_parser.xml['location'], 'location.area')
                location_country = maltego_parser.getInfoMtgl(maltego_parser.xml['location'], 'countrycode')
                location_longitude = maltego_parser.getInfoMtgl(maltego_parser.xml['location'], 'longitude')
                location_latitude = maltego_parser.getInfoMtgl(maltego_parser.xml['location'], 'latitude')
                text = f'Location:\n {location_name} \n Area:\n {location_area} \nCountry_code:\n {location_country} ' \
                       f'\nLatitude:\n {location_latitude} \nLongitude:\n {location_longitude}'

                self.createAndAddNoteToHost(host_id=host_id, name="Location Information",
                                            text=text.encode('ascii', 'ignore'))
            else:
                self.createAndAddNoteToHost(host_id=host_id, name="Location Information", text="unknown")

            if maltego_parser.xml.get('web'):
                web_name = maltego_parser.getInfoMtgl(maltego_parser.xml['web'], 'fqdn')
                text = f'Urls: \n {web_name}'
                web_port = maltego_parser.getInfoMtgl(maltego_parser.xml['web'], 'ports')
                if web_port is None:
                    web_port = 80

                web_ssh = maltego_parser.getInfoMtgl(maltego_parser.xml['web'], 'website.ssl-enabled')
                description = f'SSL Enabled: {web_ssh}'

                service_id = self.createAndAddServiceToHost(host_id=host_id, name=web_name, protocol="TCP:HTTP",
                                                            ports=web_port, description=description)

                self.createAndAddNoteToService(host_id=host_id, service_id=service_id, name="URLs",
                                               text=text.encode('ascii', 'ignore'))

            if maltego_parser.xml.get('mxrecord'):
                mx_name = maltego_parser.getInfoMtgl(maltego_parser.xml['mxrecord'], 'fqdn')

                self.createAndAddServiceToHost(host_id=host_id, name=mx_name, protocol="SMTP", ports=[25],
                                               description="E-mail Server")

            if maltego_parser.xml.get('nsrecord'):
                ns_name = maltego_parser.getInfoMtgl(maltego_parser.xml['nsrecord'], 'fqdn')
                self.createAndAddServiceToHost(host_id=host_id, name=ns_name, protocol="DNS", ports=[53],
                                               description="DNS Server")


def createPlugin(*args, **kwargs):
    return MaltegoPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
from faraday_plugins.plugins.plugin import PluginBase
import re


__author__ = "Federico Kirschbaum"
__copyright__ = "Copyright 2011, Faraday Project"
__credits__ = ["Federico Kirschbaum"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Federico Kirschbaum"
__email__ = "fedek@infobytesec.com"
__status__ = "Development"


class CmdPropeciaPlugin(PluginBase):
    """
    This plugin handles propecia command.
    Basically inserts into the tree the ouput of this tool
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "propecia"
        self.name = "propecia port scanner"
        self.plugin_version = "0.0.1"
        self.version = "1.0"
        self.framework_version = "1.0.0"
        self.options = None
        self._current_output = None
        self._command_regex = re.compile(
            r'^(sudo propecia|\.\/propecia|propecia)\s+.*?')
        self._host_ip = None
        self._port = "23"

    def parseOutputString(self, output):

        host_info = re.search(
            r"(\b(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b)", output)

        if host_info is None:
            self.logger.info("No hosts detected")
        else:
            for host in output.splitlines():
                if host != "":
                    h_id = self.createAndAddHost(host)
                    s_id = self.createAndAddServiceToHost(h_id, str(self._port), "tcp", ports=[self._port],
                                                          status="open", version="", description="")
        return True

    def processCommandString(self, username, current_path, command_string):
        super().processCommandString(username, current_path, command_string)
        count_args = command_string.split()

        if count_args.__len__() == 3:
            self._port = count_args[2]


def createPlugin(*args, **kwargs):
    return CmdPropeciaPlugin(*args, **kwargs)



from typing import List
import re

class InfoVul:
    def __init__(self, node):
        self.node = node

    @property
    def vt_id(self) -> str:
        if not self.node:
            return ''
        return self.node.get('vt_id', '')

    @property
    def request(self) -> str:
        if not self.node:
            return ''
        return self.node.get('request', '')




class Vulnerabilities:
    def __init__(self, node):
        self.node = node

    @property
    def info(self) -> InfoVul:
        return InfoVul(self.node.get('info'))

    @property
    def response(self) -> str:
        if not self.node:
            return ''
        return self.node.get('response', '')


class VulnerabilityTypes:
    def __init__(self, node):
        self.node = node

    @property
    def vt_id(self) -> str:
        if not self.node:
            return ''
        return self.node.get('vt_id', '')

    @property
    def name(self) -> str:
        if not self.node:
            return ''
        return self.node.get('name', '')

    @property
    def description(self) -> str:
        if not self.node:
            return ''
        return self.node.get('description', '')

    @property
    def severity(self) -> int:
        return self.node.get('severity', '')

    @property
    def recommendation(self) -> str:
        if not self.node:
            return ''
        return self.node.get('recommendation', '')

    @property
    def app_id(self) -> str:
        if not self.node:
            return ''
        return self.node.get('app_id', '')

    @property
    def use_ssl(self) -> bool:
        if not self.node:
            return ''
        return self.node.get('use_ssl', '')

    @property
    def tags(self) -> list:
        if not self.node:
            return ['']
        return self.node.get('tags', [''])

    def cvss_score(self) -> str:
        return self.node.get('cvss_score')

    @property
    def cvss2_vector(self) -> str:
        return self.node.get('cvss2', '')

    @property
    def cvss3_vector(self) -> str:
        return self.node.get('cvss3', '')


class Info:
    def __init__(self, node):
        self.node = node

    @property
    def host(self) -> str:
        if not self.node:
            return ''
        return self.node.get('host', '')

    @property
    def start_url(self) -> str:
        if not self.node:
            return ''
        return self.node.get('start_url', '')

class Scan:
    def __init__(self, node):
        self.node = node

    @property
    def info(self) -> Info:
        return Info(self.node.get('info'))

    @property
    def vul_types(self) -> List[VulnerabilityTypes]:
        return [VulnerabilityTypes(i) for i in self.node.get('vulnerability_types', [])]

    @property
    def vulnerabilities(self) -> List[Vulnerabilities]:
        return [Vulnerabilities(i) for i in self.node.get('vulnerabilities', [])]


class Export:
    def __init__(self, node):
        self.node = node

    @property
    def scans(self) -> List[Scan]:
        return [Scan(i) for i in self.node.get('scans', [])]

    @property
    def lang(self) -> str:
        if not self.node:
            return ''
        return self.node.get('scans', '')


class AcunetixJsonParser:
    def __init__(self, node):
        self.node = node

    @property
    def export(self) -> Export:
        return Export(self.node.get('export'))



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import re
import xml.etree.ElementTree as ET
from html.parser import HTMLParser

from faraday_plugins.plugins import plugins_utils
from faraday_plugins.plugins.plugin import PluginXMLFormat

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Facundo de Guzmán", "Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class NiktoXmlParser:
    """
    The objective of this class is to parse an xml file generated by the nikto tool.

    TODO: Handle errors.
    TODO: Test nikto output version. Handle what happens if the parser doesn't support it.
    TODO: Test cases.

    @param nikto_xml_filepath A proper xml generated by nikto
    """

    def __init__(self, xml_output):

        tree = self.parse_xml(xml_output)

        if tree:
            self.hosts = self.get_hosts(tree)
        else:
            self.hosts = []

    def parse_xml(self, xml_output):
        """
        Open and parse an xml file.

        TODO: Write custom parser to just read the nodes that we need instead of
        reading the whole file.

        @return xml_tree An xml tree instance. None if error.
        """
        try:
            tree = ET.fromstring(xml_output)
        except SyntaxError as err:
            print(f"SyntaxError: {err}. {xml_output}")
            return None

        return tree

    def get_hosts(self, tree):
        """
        @return items A list of Host instances
        """
        if tree.find('niktoscan'):
            for host_node in tree.find('niktoscan').findall('scandetails'):
                yield Host(host_node)
        else:
            for host_node in tree.findall('scandetails'):
                yield Host(host_node)


def get_attrib_from_subnode(xml_node, subnode_xpath_expr, attrib_name):
    """
    Finds a subnode in the item node and the retrieves a value from it

    @return An attribute value
    """

    node = xml_node.find(subnode_xpath_expr)

    if node is not None:
        return node.get(attrib_name)

    return None


class Item:
    """
    An abstract representation of a Item

    TODO: Consider evaluating the attributes lazily
    TODO: Write what's expected to be present in the nodes
    TODO: Refactor both Host and the Port clases?

    @param item_node A item_node taken from an nikto xml tree
    """

    def __init__(self, item_node):

        self.node = item_node

        self.osvdbid = [
            "OSVDB-ID: " + self.node.get('osvdbid')] if self.node.get('osvdbid', "0") != "0" else []

        self.namelink = self.get_text_from_subnode('namelink')
        self.iplink = self.get_text_from_subnode('iplink')

        self.id_nikto = self.node.get('id')
        self.osvdblink = self.node.get('osvdbidlink')
        self.method = self.node.get('method')

        self.uri = self.get_uri()
        self.desc = self.get_desc()
        self.params = self.get_params(self.uri)
        self.CVE = self.get_cve(self.desc)

    def get_uri(self):

        try:

            uri = self.get_text_from_subnode('uri')
            h = HTMLParser.HTMLParser()
            return h.unescape(uri)

        except Exception as e:
            return uri

    def get_desc(self):

        desc = self.get_text_from_subnode('description')

        try:

            uri_present = desc.split(': ', 1)[0]
            h = HTMLParser.HTMLParser()
            if uri_present == h.unescape(self.uri):

                name = desc.split(': ', 1)[1]
                if name is not None and name != '':
                    return name

            return desc

        except Exception as e:
            return desc

    def get_params(self, uri):
        """Return the paramethers as a string"""
        try:
            params = uri.split('?')[1].replace('&', ',')
        except Exception as e:
            params = ''

        return params

    def get_text_from_subnode(self, subnode_xpath_expr):
        """
        Finds a subnode in the host node and the retrieves a value from it.

        @return An attribute value
        """
        sub_node = self.node.find(subnode_xpath_expr)
        if sub_node is not None:
            return sub_node.text

        return None

    def get_cve(self, desc):
        match = plugins_utils.CVE_regex.search(desc)
        if match:
            return [match.group()]
        return []


class Host:
    """
    An abstract representation of a Host

    @param host_node A host_node taken from an nmap xml tree
    """

    def __init__(self, host_node):

        self.node = host_node
        self.targetip = self.node.get('targetip')
        self.targethostname = self.node.get('targethostname')
        self.port = self.node.get('targetport')
        self.targetbanner = self.node.get('targetbanner')
        self.starttime = self.node.get('starttime')
        self.sitename = self.node.get('sitename')
        self.siteip = self.node.get('hostheader')
        self.items = self.get_items()

    def get_items(self):
        """
        @return items A list of Host instances
        """
        for item_node in self.node.findall('item'):
            yield Item(item_node)



class NiktoPlugin(PluginXMLFormat):
    """
    Example plugin to parse nikto output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = ["niktoscan", "niktoscans"]
        self.id = "Nikto"
        self.name = "Nikto XML Output Plugin"
        self.plugin_version = "0.0.2"
        self.version = "2.1.5"
        self.options = None
        self.parent = None
        self._use_temp_file = True
        self._temp_file_extension = "xml"
        self.xml_arg_re = re.compile(r"^.*(-output\s*[^\s]+).*$")
        self._command_regex = re.compile(
            r'^(sudo nikto|nikto|sudo nikto\.pl|nikto\.pl|perl nikto\.pl|\.\/nikto\.pl|\.\/nikto)\s+.*?')
        self._completition = {
            "": "",
            "-ask+": "Whether to ask about submitting updates",
            "-Cgidirs+": 'Scan these CGI dirs: "none", "all", or values like "/cgi/ /cgi-a/"',
            "-config+": "Use this config file",
            "-Display+": "Turn on/off display outputs:",
            "-dbcheck": "Check database and other key files for syntax errors",
            "-evasion+": "Encoding technique:",
            "-Format+": "Save file (-o) format:",
            "-Help": "Extended help information",
            "-host+": "Target host",
            "-IgnoreCode": "Ignore Codes--treat as negative responses",
            "-id+": "Host authentication to use, format is id:pass or id:pass:realm",
            "-key+": "Client certificate key file",
            "-list-plugins": "List all available plugins, perform no testing",
            "-maxtime+": "Maximum testing time per host",
            "-mutate+": "Guess additional file names:",
            "-mutate-options": "Provide information for mutates",
            "-nointeractive": "Disables interactive features",
            "-nolookup": "Disables DNS lookups",
            "-nossl": "Disables the use of SSL",
            "-no404": "Disables nikto attempting to guess a 404 page",
            "-output+": "Write output to this file ('.' for auto-name)",
            "-Pause+": "Pause between tests (seconds, integer or float)",
            "-Plugins+": "List of plugins to run (default: ALL)",
            "-port+": "Port to use (default 80)",
            "-RSAcert+": "Client certificate file",
            "-root+": "Prepend root value to all requests, format is /directory",
            "-Save": "Save positive responses to this directory ('.' for auto-name)",
            "-ssl": "Force ssl mode on port",
            "-Tuning+": "Scan tuning:",
            "-timeout+": "Timeout for requests (default 10 seconds)",
            "-Userdbs": "Load only user databases, not the standard databases",
            "-until": "Run until the specified time or duration",
            "-update": "Update databases and plugins from CIRT.net",
            "-useproxy": "Use the proxy defined in nikto.conf",
            "-Version": "Print plugin and database versions",
            "-vhost+": "Virtual host (for Host header)",
        }

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it from
        the xml where it expects it to be present.

        NOTE: if 'debug' is true then it is being run from a test case and the
        output being sent is valid.
        """

        parser = NiktoXmlParser(output)

        for host in parser.hosts:

            h_id = self.createAndAddHost(
                host.targetip,
                hostnames=[host.targethostname]
            )

            s_id = self.createAndAddServiceToHost(
                h_id,
                "http",
                "tcp",
                ports=[host.port],
                status="open"
            )

            for item in host.items:
                self.createAndAddVulnWebToService(
                    h_id,
                    s_id,
                    name=item.desc,
                    ref=item.osvdbid,
                    method=item.method,
                    params=', '.join(item.params),
                    cve=item.CVE,
                    **plugins_utils.get_vulnweb_url_fields(item.namelink)
                )

        del parser

    def processCommandString(self, username, current_path, command_string):
        """
        Adds the -oX parameter to get xml output to the command string that the
        user has set.
        """
        super().processCommandString(username, current_path, command_string)

        arg_match = self.xml_arg_re.match(command_string)

        if arg_match is None:
            return re.sub(r"(^.*?nikto(\.pl)?)", r"\1 -output %s -Format XML" % self._output_file_path, command_string)
        else:
            data = re.sub(r" \-Format XML", "", command_string)
            return re.sub(arg_match.group(1), r"-output %s -Format XML" % self._output_file_path, data)


def createPlugin(*args, **kwargs):
    return NiktoPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
from faraday_plugins.plugins.plugin import PluginBase
import re

__author__ = "Federico Kirschbaum"
__copyright__ = "Copyright 2013, Faraday Project"
__credits__ = ["Federico Kirschbaum"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Federico Kirschbaum"
__email__ = "fedek@infobytesec.com"
__status__ = "Development"


class CmdArpScanPlugin(PluginBase):
    """
    This plugin handles arp-scan command.
    Basically inserts into the tree the ouput of this tool
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "arp-scan"
        self.name = "arp-scan network scanner"
        self.plugin_version = "0.0.2"
        self.version = "1.8.1"
        self.framework_version = "1.0.0"
        self.options = None
        self._current_output = None
        self._command_regex = re.compile(r'^(sudo arp-scan|\.\/arp-scan|arp-scan)\s+.*?')
        self._host_ip = None

    def parseOutputString(self, output):

        host_info = re.search(
            r"(\b(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b)",
            output)

        host_mac_addr = re.search(r"([\dA-F]{2}(?:[-:][\dA-F]{2}){5})", output, re.IGNORECASE)

        if host_info is None:
            self.logger.info("No hosts detected")
        else:

            for line in output.split('\n'):
                vals = line.split("\t")

                if len(vals) == 3:

                    if len(vals[0].split(".")) == 4:

                        host = vals[0]
                        h_id = self.createAndAddHost(host, mac=vals[1])

        return True



def createPlugin(*args, **kwargs):
    return CmdArpScanPlugin(*args, **kwargs)



import re
from collections import defaultdict

from faraday_plugins.plugins.plugin import PluginBase

class RDPScanPlugin(PluginBase):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = "rdpscan"
        self.id = "rdpscan"
        self.name = "rdpscan"
        self._command_regex = re.compile(r'^(sudo rdpscan|rdpscan|\.\/rdpscan)\s+.*?')

    def parseOutputString(self, output):
        services = defaultdict(set)
        for info in output.split('\n'):
            if info:
                ip, status, data = info.split('-', 2)
                ip = ip.strip()
                status = status.strip()
                data = data.strip()
                if status.lower() == 'unknown':
                    continue

                host_id = self.createAndAddHost(ip)
                service_id = self.createAndAddServiceToHost(
                    host_id=host_id,
                    name='rdp',
                    ports=3389,
                    protocol='tcp',
                )
                if status.lower() == 'vulnerable':
                    description = "A remote code execution vulnerability exists in Remote Desktop Services formerly known as Terminal Services when an unauthenticated attacker connects to the target system using RDP and sends specially crafted requests, aka 'Remote Desktop Services Remote Code Execution Vulnerability'. "
                    self.createAndAddVulnToService(
                        host_id=host_id,
                        service_id=service_id,
                        name='Remote Desktop Services Remote Code Execution Vulnerability',
                        desc=description,
                        ref=['CVE-2019-0708']
                    )


def createPlugin(*args, **kwargs):
    return RDPScanPlugin(*args, **kwargs)
